"""
Tools for modeling Scheme datastructures used in LilyPond.
"""

import functools
import typing
from abjad import enums
from abjad import typings
from abjad.mathtools.NonreducedFraction import NonreducedFraction
from abjad.system.AbjadValueObject import AbjadValueObject
from abjad.system.FormatSpecification import FormatSpecification
from abjad.system.FormatSpecification import FormatSpecification
from abjad.system.StorageFormatManager import StorageFormatManager
from abjad.utilities.String import String



class Scheme(AbjadValueObject):
    r"""
    Abjad model of Scheme code.

    ..  container:: example

        A Scheme boolean value:

        >>> scheme = abjad.Scheme(True)
        >>> print(format(scheme))
        ##t

    ..  container:: example

        A nested Scheme expession:

        >>> scheme = abjad.Scheme([
        ...     ('left', (1, 2, False)),
        ...     ('right', (1, 2, 3.3)),
        ...     ])
        >>> print(format(scheme))
        #((left (1 2 #f)) (right (1 2 3.3)))

    ..  container:: example

        A list:

        >>> scheme_1 = abjad.Scheme([1, 2, 3])
        >>> scheme_2 = abjad.Scheme((1, 2, 3))
        >>> format(scheme_1) == format(scheme_2)
        True

        Scheme wraps nested variable-length arguments in a tuple.

    ..  container:: example

        A quoted Scheme expression:

        >>> scheme = abjad.Scheme((1, 2, 3), quoting="'#")
        >>> print(format(scheme))
        #'#(1 2 3)

        Use the ``quoting`` keyword to prepend Scheme's various quote, unquote,
        unquote-splicing characters to formatted output.

    ..  container:: example

        A Scheme expression with forced quotes:

        >>> scheme = abjad.Scheme('nospaces', force_quotes=True)
        >>> print(format(scheme))
        #"nospaces"

        Use this in certain \override situations when LilyPond's Scheme
        interpreter treats unquoted strings as symbols instead of strings.
        The string must contain no whitespace for this to work.

    ..  container:: example

        A Scheme expression of LilyPond functions:

        >>> function_1 = 'tuplet-number::append-note-wrapper'
        >>> function_2 = 'tuplet-number::calc-denominator-text'
        >>> string = abjad.Scheme('4', force_quotes=True)
        >>> scheme = abjad.Scheme([function_1, function_2, string])
        >>> abjad.f(scheme)
        abjad.Scheme(
            [
                'tuplet-number::append-note-wrapper',
                'tuplet-number::calc-denominator-text',
                abjad.Scheme(
                    '4',
                    force_quotes=True,
                    ),
                ]
            )

        >>> print(format(scheme))
        #(tuplet-number::append-note-wrapper tuplet-number::calc-denominator-text "4")

    ..  container:: example

        A Scheme lambda expression of LilyPond function that takes a markup
        with a quoted string argument. Setting verbatim to true causes the
        expression to format exactly as-is without modifying quotes or
        whitespace:

        >>> string = '(lambda (grob) (grob-interpret-markup grob'
        >>> string += r' #{ \markup \musicglyph #"noteheads.s0harmonic" #}))'
        >>> scheme = abjad.Scheme(string, verbatim=True)
        >>> abjad.f(scheme)
        abjad.Scheme(
            '(lambda (grob) (grob-interpret-markup grob #{ \\markup \\musicglyph #"noteheads.s0harmonic" #}))',
            verbatim=True,
            )

        >>> print(format(scheme))
        #(lambda (grob) (grob-interpret-markup grob #{ \markup \musicglyph #"noteheads.s0harmonic" #}))

    ..  container:: example

        More examples:

        >>> abjad.Scheme(True)
        Scheme(True)

        >>> abjad.Scheme(False)
        Scheme(False)

        >>> abjad.Scheme(None)
        Scheme(None)

        >>> abjad.Scheme('hello')
        Scheme('hello')

        >>> abjad.Scheme('hello world')
        Scheme('hello world')

            >>> abjad.Scheme([abjad.Scheme('foo'), abjad.Scheme(3.14159)])
            Scheme([Scheme('foo'), Scheme(3.14159)])

            >>> abjad.Scheme([
            ...     abjad.SchemePair(('padding', 1)),
            ...     abjad.SchemePair(('attach-dir', -1)),
            ...     ])
            Scheme([SchemePair(('padding', 1)), SchemePair(('attach-dir', -1))])

    ..  container:: example

        Scheme takes an optional ``quoting`` keyword, for prepending
        quote/unquote ticks:

        >>> str(abjad.Scheme(['fus', 'ro', 'dah'], quoting = "',"))
        "',(fus ro dah)"

    ..  container:: example

        __str__ of abjad.Scheme returns the abjad.Scheme formatted value
        without the hash mark, while format(Scheme) returns the formatted value
        with the hash mark, allowing for nested abjad.Scheme expressions:

        >>> scheme = abjad.Scheme(['fus', 'ro', 'dah'], quoting = "'")
        >>> str(scheme)
        "'(fus ro dah)"

        >>> format(scheme)
        "#'(fus ro dah)"

    ..  container:: example

        Scheme attempts to format Python values into abjad.Scheme equivalents:

        >>> format(abjad.Scheme(True))
        '##t'

        >>> format(abjad.Scheme(False))
        '##f'

        >>> format(abjad.Scheme(None))
        '##f'

        >>> format(abjad.Scheme('hello world'))
        '#"hello world"'

        >>> format(abjad.Scheme([1, 2, 3]))
        '#(1 2 3)'

        >>> format(abjad.Scheme([
        ...     abjad.SchemePair(('padding', 1)),
        ...     abjad.SchemePair(('attach-dir', -1)),
        ...     ],
        ...     quoting="'",
        ...     ))
        "#'((padding . 1) (attach-dir . -1))"

    """

    ### CLASS VARIABLES ###

    __slots__ = (
        '_force_quotes',
        '_quoting',
        '_value',
        '_verbatim',
        )

    _publish_storage_format = True

    lilypond_color_constants = (
        'black',
        'blue',
        'center',
        'cyan',
        'darkblue',
        'darkcyan',
        'darkgreen',
        'darkmagenta',
        'darkred',
        'darkyellow',
        'down',
        'green',
        'grey',
        'left',
        'magenta',
        'red',
        'right',
        'up',
        'white',
        'yellow',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        value: typing.Any = None,
        force_quotes: bool = None,
        quoting: str = None,
        verbatim: bool = None,
        ) -> None:
        self._value = value
        if force_quotes is not None:
            force_quotes = bool(force_quotes)
        self._force_quotes = force_quotes
        if quoting is not None and not set(r"',@`#").issuperset(set(quoting)):
            message = rf"quoting must be ' or , or @ or ` or #: {quoting!r}."
            raise ValueError(message)
        self._quoting = quoting
        if verbatim is not None:
            verbatim = bool(verbatim)
        self._verbatim = verbatim

    ### SPECIAL METHODS ###

    def __format__(self, format_specification='') -> str:
        """
        Formats scheme.

        ..  container:: example

            Scheme LilyPond format:

            >>> scheme = abjad.Scheme('foo')
            >>> format(scheme)
            '#foo'

        ..  container:: example

            Scheme storage format:

            >>> abjad.f(scheme)
            abjad.Scheme(
                'foo'
                )

        """
        if format_specification in ('', 'lilypond'):
            return self._get_lilypond_format()
        assert format_specification == 'storage'
        return StorageFormatManager(self).get_storage_format()

    def __str__(self) -> str:
        """
        Gets string representation of Scheme object.
        """
        string = self._get_formatted_value()
        if self.quoting is not None:
            string = self.quoting + string
        return string

    ### PRIVATE METHODS ###

    def _get_format_specification(self):
        values = [self.value]
        return FormatSpecification(
            client=self,
            storage_format_args_values=values,
            )

    def _get_formatted_value(self):
        return Scheme.format_scheme_value(
            self.value,
            force_quotes=self.force_quotes,
            verbatim=self.verbatim,
            )

    def _get_lilypond_format(self):
        string = self._get_formatted_value()
        if self.quoting is not None:
            string = self.quoting + string
        if string in ('#t', '#f') or not string.startswith('#'):
            string = '#' + string
        return string

    ### PUBLIC PROPERTIES ###

    @property
    def force_quotes(self) -> typing.Optional[bool]:
        """
        Is true when quotes should be forced in output.
        """
        return self._force_quotes

    @property
    def quoting(self) -> typing.Optional[str]:
        """
        Gets Scheme quoting string.
        """
        return self._quoting

    @property
    def value(self) -> typing.Any:
        """
        Gets value.
        """
        return self._value

    @property
    def verbatim(self) -> typing.Optional[bool]:
        """
        Is true when formatting should format value absolutely verbatim.
        Whitespace, quotes, and all other parts of value are left intact.
        """
        return self._verbatim

    ### PUBLIC METHODS ###

    @staticmethod
    def format_embedded_scheme_value(
        value: typing.Any,
        force_quotes: bool = False,
        ) -> str:
        """
        Formats embedded Scheme ``value``.
        """
        if isinstance(value, (enums.HorizontalAlignment, enums.VerticalAlignment)):
            return '#' + repr(value).lower()
        result = Scheme.format_scheme_value(value, force_quotes=force_quotes)
        if isinstance(value, bool):
            result = '#' + result
        elif isinstance(value, str) and value.startswith('#'):
            pass
        elif isinstance(value, str) and not force_quotes:
            result = '#' + result
        elif isinstance(value, Scheme):
            result = '#' + result
        return result

    @staticmethod
    def format_scheme_value(
        value: typing.Any,
        force_quotes: bool = False,
        verbatim: bool = False,
        ) -> str:
        r"""
        Formats ``value`` as Scheme would.

        ..  container:: example

            Some basic values:

            >>> abjad.Scheme.format_scheme_value(1)
            '1'

            >>> abjad.Scheme.format_scheme_value('foo')
            'foo'

            >>> abjad.Scheme.format_scheme_value('bar baz')
            '"bar baz"'

            >>> abjad.Scheme.format_scheme_value([1.5, True, False])
            '(1.5 #t #f)'

        ..  container:: example

            Strings without whitespace can be forcibly quoted via the
            ``force_quotes`` keyword:

            >>> abjad.Scheme.format_scheme_value(
            ...     'foo',
            ...     force_quotes=True,
            ...     )
            '"foo"'

        ..  container:: example

            Set verbatim to true to format value exactly (with only hash
            preprended):

            >>> string = '(lambda (grob) (grob-interpret-markup grob'
            >>> string += r' #{ \markup \musicglyph #"noteheads.s0harmonic" #}))'
            >>> abjad.Scheme.format_scheme_value(string, verbatim=True)
            '(lambda (grob) (grob-interpret-markup grob #{ \\markup \\musicglyph #"noteheads.s0harmonic" #}))'

        ..  container:: example

            Hash symbol at the beginning of a string does not result in quoted
            output:

            >>> string = '#1-finger'
            >>> abjad.Scheme.format_scheme_value(string)
            '#1-finger'

        """
        if isinstance(value, str) and verbatim:
            return value
        elif isinstance(value, str) and not verbatim:
            value = value.replace('"', r'\"')
            if value.startswith('#'):
                pass
            elif force_quotes or ' ' in value or '#' in value:
                return f'"{value}"'
            return value
        elif value is True:
            return '#t'
        elif value is False:
            return '#f'
        elif isinstance(value, (list, tuple)):
            string = ' '.join(Scheme.format_scheme_value(_) for _ in value)
            return f'({string})'
        elif isinstance(value, Scheme):
            return str(value)
        elif value is None:
            return '#f'
        return str(value)


class SchemeAssociativeList(Scheme):
    """
    Abjad model of Scheme associative list.

    ..  container:: example

        >>> scheme_alist = abjad.SchemeAssociativeList([
        ...     ('space', 2),
        ...     ('padding', 0.5),
        ...     ])
        >>> abjad.f(scheme_alist)
        abjad.SchemeAssociativeList(
            [
                abjad.SchemePair(('space', 2)),
                abjad.SchemePair(('padding', 0.5)),
                ]
            )

        >>> print(format(scheme_alist))
        #'((space . 2) (padding . 0.5))

    Scheme associative lists are immutable.
    """

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(
        self,
        value: typing.List = None,
        ) -> None:
        value = value or []
        pairs = []
        for item in value:
            if isinstance(item, tuple):
                pair = SchemePair(item)
            elif isinstance(item, SchemePair):
                pair = item
            else:
                message = f'must be Python pair or Scheme pair: {item!r}.'
                raise TypeError(message)
            pairs.append(pair)
        Scheme.__init__(self, value=pairs, quoting="'")


class SchemeColor(Scheme):
    r"""
    Abjad model of Scheme color.

    ..  container:: example

        >>> abjad.SchemeColor('ForestGreen')
        SchemeColor('ForestGreen')


    ..  container:: example

        >>> note = abjad.Note("c'4")
        >>> scheme_color = abjad.SchemeColor('ForestGreen')
        >>> abjad.override(note).note_head.color = scheme_color
        >>> abjad.show(note) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(note)
            \once \override NoteHead.color = #(x11-color 'ForestGreen)
            c'4

    """

    ### CLASS VARIABLES ##

    __slots__ = ()

    ### PRIVATE METHODS ###

    def _get_formatted_value(self):
        string = "(x11-color '{})"
        string = string.format(self._value)
        return string


@functools.total_ordering
class SchemeMoment(Scheme):
    """
    Abjad model of Scheme moment.

    ..  container:: example

        Initializes with two integers:

        >>> abjad.SchemeMoment((2, 68))
        SchemeMoment((2, 68))

    Scheme moments are immutable.
    """

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(
        self,
        duration: typing.Union[typing.Tuple[int, int]] = (0, 1),
        ) -> None:
        pair = NonreducedFraction(duration).pair
        Scheme.__init__(self, pair)

    ### SPECIAL METHODS ###

    def __eq__(self, argument) -> bool:
        """
        Is true when ``argument`` is a scheme moment with the same value as
        that of this scheme moment.

        ..  container:: example

            >>> abjad.SchemeMoment((2, 68)) == abjad.SchemeMoment((2, 68))
            True

        ..  container:: example

            Otherwise false:

            >>> abjad.SchemeMoment((2, 54)) == abjad.SchemeMoment((2, 68))
            False

        """
        return super().__eq__(argument)

    def __hash__(self) -> int:
        """
        Hashes scheme moment.

        Redefined in tandem with ``__eq__``.
        """
        return super().__hash__()

    def __lt__(self, argument) -> bool:
        """
        Is true when ``argument`` is a scheme moment with value greater than
        that of this scheme moment.

        ..  container:: example

            >>> abjad.SchemeMoment((1, 68)) < abjad.SchemeMoment((1, 32))
            True

        ..  container:: example

            Otherwise false:

            >>> abjad.SchemeMoment((1, 68)) < abjad.SchemeMoment((1, 78))
            False

        """
        if isinstance(argument, type(self)):
            if self.duration < argument.duration:
                return True
        return False

    ### PRIVATE METHODS ###

    def _get_format_specification(self):
        values = [self.value]
        return FormatSpecification(
            client=self,
            storage_format_args_values=values,
            storage_format_kwargs_names=[],
            )

    def _get_formatted_value(self):
        pair = self.duration.pair
        string = f'(ly:make-moment {pair[0]} {pair[1]})'
        return string

    ### PUBLIC PROPERTIES ###

    @property
    def duration(self) -> NonreducedFraction:
        """
        Gets duration of Scheme moment.

        ..  container:: example

            >>> abjad.SchemeMoment((2, 68)).duration
            NonreducedFraction(2, 68)

        """
        return NonreducedFraction(self.value)


class SchemePair(Scheme):
    r"""
    Abjad model of Scheme pair.

    ..  container:: example

        Initializes from two values:

        >>> abjad.SchemePair(('spacing', 4))
        SchemePair(('spacing', 4))

    ..  container:: example

        REGRESSION:

        Right-hand side string forces quotes:

        >>> scheme_pair = abjad.SchemePair(('font-name', 'Times'))
        >>> format(scheme_pair)
        '#\'(font-name . "Times")'

        Right-hand side nonstring does not force quotes:

        >>> scheme_pair = abjad.SchemePair(('spacing', 4))
        >>> format(scheme_pair)
        "#'(spacing . 4)"

    """

    ### CLASS VARIABLES ##

    __slots__ = (
        '_value',
        )

    ### INITIALIZER ##

    def __init__(
        self,
        value = (None, None),
        ) -> None:
        assert isinstance(value, tuple), repr(value)
        assert len(value) == 2, repr(value)
        Scheme.__init__(self, value=value)

    ### SPECIAL METHODS ###

    def __format__(self, format_specification='') -> str:
        """
        Formats Scheme pair.

        ..  container:: example

            >>> scheme_pair = abjad.SchemePair((-1, 1))

            >>> format(scheme_pair)
            "#'(-1 . 1)"

            >>> abjad.f(scheme_pair)
            abjad.SchemePair((-1, 1))

        """
        return super().__format__(format_specification=format_specification)

    ### PRIVATE METHODS ###

    def _get_format_specification(self):
        values = [self.value]
        return FormatSpecification(
            client=self,
            repr_is_indented=False,
            storage_format_is_indented=False,
            storage_format_args_values=values,
            )

    def _get_formatted_value(self):
        assert len(self._value) == 2
        lhs = Scheme.format_scheme_value(self._value[0])
        # need to force quotes around pairs like
        # \override #'(font-name . "Times")
        rhs = Scheme.format_scheme_value(
            self._value[-1],
            force_quotes=True,
            )
        return f'({lhs} . {rhs})'

    def _get_lilypond_format(self):
        string = self._get_formatted_value()
        return f"#'{string}"

    ### PUBLIC PROPERTIES ###

    @property
    def left(self) -> typing.Any:
        """
        Gets left value.
        """
        pair = self.value
        assert isinstance(pair, tuple)
        return pair[0]

    @property
    def right(self) -> typing.Any:
        """
        Gets right value.
        """
        pair = self.value
        assert isinstance(pair, tuple)
        return pair[-1]


class SchemeSymbol(Scheme):
    """
    Abjad model of Scheme symbol.

    ..  container:: example

        >>> scheme = abjad.SchemeSymbol('cross')
        >>> scheme
        SchemeSymbol('cross')

        >>> print(format(scheme))
        #'cross

    """

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(
        self,
        symbol: str = 'cross',
        ) -> None:
        symbol = str(symbol)
        Scheme.__init__(self, value=symbol, quoting="'")

    ### PRIVATE METHODS ###

    def _get_format_specification(self):
        values = [self.symbol]
        return FormatSpecification(
            client=self,
            storage_format_args_values=values,
            storage_format_kwargs_names=[],
            )

    ### PUBLIC PROPERTIES ###

    @property
    def symbol(self) -> str:
        """
        Gets symbol string.
        """
        assert isinstance(self.value, str)
        return self.value


class SchemeVector(Scheme):
    """
    Abjad model of Scheme vector.

    ..  container:: example

        Scheme vector of boolean values:

        >>> scheme = abjad.SchemeVector([True, True, False])
        >>> scheme
        SchemeVector(True, True, False)
        >>> print(format(scheme))
        #'(#t #t #f)

    ..  container:: example

        Scheme vector of symbols:

        >>> scheme = abjad.SchemeVector(['foo', 'bar', 'blah'])
        >>> scheme
        SchemeVector('foo', 'bar', 'blah')
        >>> print(format(scheme))
        #'(foo bar blah)

    Scheme vectors and Scheme vector constants differ in only their LilyPond
    input format.
    """

    ### CLASS VARIABLES ##

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(
        self,
        value: typing.List = [],
        ) -> None:
        Scheme.__init__(self, value, quoting="'")

    ### PRIVATE METHODS ###

    def _get_format_specification(self):
        values = self._value
        if String.is_string(self._value):
            values = [self._value]
        return FormatSpecification(
            client=self,
            storage_format_args_values=values,
            storage_format_kwargs_names=[],
            )


class SchemeVectorConstant(Scheme):
    """
    Abjad model of Scheme vector constant.

    ..  container:: example

        Scheme vector constant of boolean values:

        >>> scheme = abjad.SchemeVectorConstant([True, True, False])
        >>> scheme
        SchemeVectorConstant(True, True, False)
        >>> print(format(scheme))
        #'#(#t #t #f)

    Scheme vectors and Scheme vector constants differ in only their LilyPond
    input format.
    """

    ### CLASS VARIABLES ##

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(
        self,
        value: typing.List = [],
        ) -> None:
        Scheme.__init__(self, value, quoting="'#")

    ### PRIVATE METHODS ###

    def _get_format_specification(self):
        values = self._value
        if String.is_string(self._value):
            values = [self._value]
        return FormatSpecification(
            client=self,
            storage_format_args_values=values,
            storage_format_kwargs_names=[],
            )


class SpacingVector(SchemeVector):
    r"""
    Abjad model of Scheme spacing vector.

    ..  container:: example

        >>> vector = abjad.SpacingVector(0, 0, 12, 0)

        >>> abjad.f(vector)
        abjad.SpacingVector(
            abjad.SchemePair(('basic-distance', 0)),
            abjad.SchemePair(('minimum-distance', 0)),
            abjad.SchemePair(('padding', 12)),
            abjad.SchemePair(('stretchability', 0))
            )

        Use to set paper block spacing attributes:

        >>> staff = abjad.Staff("c'8 d'8 e'8 f'8")
        >>> lilypond_file = abjad.LilyPondFile.new(staff)
        >>> vector = abjad.SpacingVector(0, 0, 12, 0)
        >>> lilypond_file.paper_block.system_system_spacing = vector

        ..  docs::

            >>> abjad.f(lilypond_file.paper_block)
            \paper {
                system-system-spacing = #'((basic-distance . 0) (minimum-distance . 0) (padding . 12) (stretchability . 0))
            }

    """

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(
        self,
        basic_distance: typings.Number = 0,
        minimum_distance: typings.Number = 0,
        padding: typings.Number = 12,
        stretchability: typings.Number = 0,
        ) -> None:
        pairs = [
            SchemePair(('basic-distance', basic_distance)),
            SchemePair(('minimum-distance', minimum_distance)),
            SchemePair(('padding', padding)),
            SchemePair(('stretchability', stretchability)),
            ]
        return SchemeVector.__init__(self, pairs)



lilypond_version = "2.19.24"

music_glyphs = set([
    "accidentals.doublesharp",
    "accidentals.flat",
    "accidentals.flat.arrowboth",
    "accidentals.flat.arrowdown",
    "accidentals.flat.arrowup",
    "accidentals.flat.slash",
    "accidentals.flat.slashslash",
    "accidentals.flatflat",
    "accidentals.flatflat.slash",
    "accidentals.hufnagelM1",
    "accidentals.kievan1",
    "accidentals.kievanM1",
    "accidentals.leftparen",
    "accidentals.medicaeaM1",
    "accidentals.mensural1",
    "accidentals.mensuralM1",
    "accidentals.mirroredflat",
    "accidentals.mirroredflat.backslash",
    "accidentals.mirroredflat.flat",
    "accidentals.natural",
    "accidentals.natural.arrowboth",
    "accidentals.natural.arrowdown",
    "accidentals.natural.arrowup",
    "accidentals.rightparen",
    "accidentals.sharp",
    "accidentals.sharp.arrowboth",
    "accidentals.sharp.arrowdown",
    "accidentals.sharp.arrowup",
    "accidentals.sharp.slashslash.stem",
    "accidentals.sharp.slashslash.stemstemstem",
    "accidentals.sharp.slashslashslash.stem",
    "accidentals.sharp.slashslashslash.stemstem",
    "accidentals.vaticana0",
    "accidentals.vaticanaM1",
    "accordion.bayanbass",
    "accordion.discant",
    "accordion.dot",
    "accordion.freebass",
    "accordion.oldEE",
    "accordion.pull",
    "accordion.push",
    "accordion.stdbass",
    "arrowheads.close.01",
    "arrowheads.close.0M1",
    "arrowheads.close.11",
    "arrowheads.close.1M1",
    "arrowheads.open.01",
    "arrowheads.open.0M1",
    "arrowheads.open.11",
    "arrowheads.open.1M1",
    "brackettips.down",
    "brackettips.up",
    "clefs.C",
    "clefs.C_change",
    "clefs.F",
    "clefs.F_change",
    "clefs.G",
    "clefs.GG",
    "clefs.GG_change",
    "clefs.G_change",
    "clefs.blackmensural.c",
    "clefs.blackmensural.c_change",
    "clefs.hufnagel.do",
    "clefs.hufnagel.do.fa",
    "clefs.hufnagel.do.fa_change",
    "clefs.hufnagel.do_change",
    "clefs.hufnagel.fa",
    "clefs.hufnagel.fa_change",
    "clefs.kievan.do",
    "clefs.kievan.do_change",
    "clefs.medicaea.do",
    "clefs.medicaea.do_change",
    "clefs.medicaea.fa",
    "clefs.medicaea.fa_change",
    "clefs.mensural.c",
    "clefs.mensural.c_change",
    "clefs.mensural.f",
    "clefs.mensural.f_change",
    "clefs.mensural.g",
    "clefs.mensural.g_change",
    "clefs.neomensural.c",
    "clefs.neomensural.c_change",
    "clefs.percussion",
    "clefs.percussion_change",
    "clefs.petrucci.c1",
    "clefs.petrucci.c1_change",
    "clefs.petrucci.c2",
    "clefs.petrucci.c2_change",
    "clefs.petrucci.c3",
    "clefs.petrucci.c3_change",
    "clefs.petrucci.c4",
    "clefs.petrucci.c4_change",
    "clefs.petrucci.c5",
    "clefs.petrucci.c5_change",
    "clefs.petrucci.f",
    "clefs.petrucci.f_change",
    "clefs.petrucci.g",
    "clefs.petrucci.g_change",
    "clefs.tab",
    "clefs.tab_change",
    "clefs.tenorG",
    "clefs.tenorG_change",
    "clefs.varC",
    "clefs.varC_change",
    "clefs.varpercussion",
    "clefs.varpercussion_change",
    "clefs.vaticana.do",
    "clefs.vaticana.do_change",
    "clefs.vaticana.fa",
    "clefs.vaticana.fa_change",
    "comma",
    "custodes.hufnagel.d0",
    "custodes.hufnagel.d1",
    "custodes.hufnagel.d2",
    "custodes.hufnagel.u0",
    "custodes.hufnagel.u1",
    "custodes.hufnagel.u2",
    "custodes.medicaea.d0",
    "custodes.medicaea.d1",
    "custodes.medicaea.d2",
    "custodes.medicaea.u0",
    "custodes.medicaea.u1",
    "custodes.medicaea.u2",
    "custodes.mensural.d0",
    "custodes.mensural.d1",
    "custodes.mensural.d2",
    "custodes.mensural.u0",
    "custodes.mensural.u1",
    "custodes.mensural.u2",
    "custodes.vaticana.d0",
    "custodes.vaticana.d1",
    "custodes.vaticana.d2",
    "custodes.vaticana.u0",
    "custodes.vaticana.u1",
    "custodes.vaticana.u2",
    "dots.dot",
    "dots.dotkievan",
    "dots.dotvaticana",
    "eight",
    "f",
    "five",
    "flags.d3",
    "flags.d4",
    "flags.d5",
    "flags.d6",
    "flags.d7",
    "flags.dgrace",
    "flags.mensurald03",
    "flags.mensurald04",
    "flags.mensurald05",
    "flags.mensurald06",
    "flags.mensurald13",
    "flags.mensurald14",
    "flags.mensurald15",
    "flags.mensurald16",
    "flags.mensurald23",
    "flags.mensurald24",
    "flags.mensurald25",
    "flags.mensurald26",
    "flags.mensuralu03",
    "flags.mensuralu04",
    "flags.mensuralu05",
    "flags.mensuralu06",
    "flags.mensuralu13",
    "flags.mensuralu14",
    "flags.mensuralu15",
    "flags.mensuralu16",
    "flags.mensuralu23",
    "flags.mensuralu24",
    "flags.mensuralu25",
    "flags.mensuralu26",
    "flags.u3",
    "flags.u4",
    "flags.u5",
    "flags.u6",
    "flags.u7",
    "flags.ugrace",
    "four",
    "hyphen",
    "m",
    "nine",
    "noteheads.d0doFunk",
    "noteheads.d0fa",
    "noteheads.d0faFunk",
    "noteheads.d0faThin",
    "noteheads.d0miFunk",
    "noteheads.d0reFunk",
    "noteheads.d0tiFunk",
    "noteheads.d1do",
    "noteheads.d1doFunk",
    "noteheads.d1doThin",
    "noteheads.d1doWalker",
    "noteheads.d1fa",
    "noteheads.d1faFunk",
    "noteheads.d1faThin",
    "noteheads.d1faWalker",
    "noteheads.d1miFunk",
    "noteheads.d1re",
    "noteheads.d1reFunk",
    "noteheads.d1reThin",
    "noteheads.d1reWalker",
    "noteheads.d1ti",
    "noteheads.d1tiFunk",
    "noteheads.d1tiThin",
    "noteheads.d1tiWalker",
    "noteheads.d1triangle",
    "noteheads.d2do",
    "noteheads.d2doFunk",
    "noteheads.d2doThin",
    "noteheads.d2doWalker",
    "noteheads.d2fa",
    "noteheads.d2faFunk",
    "noteheads.d2faThin",
    "noteheads.d2faWalker",
    "noteheads.d2kievan",
    "noteheads.d2re",
    "noteheads.d2reFunk",
    "noteheads.d2reThin",
    "noteheads.d2reWalker",
    "noteheads.d2ti",
    "noteheads.d2tiFunk",
    "noteheads.d2tiThin",
    "noteheads.d2tiWalker",
    "noteheads.d2triangle",
    "noteheads.d3kievan",
    "noteheads.dM2",
    "noteheads.dM2blackmensural",
    "noteheads.dM2mensural",
    "noteheads.dM2neomensural",
    "noteheads.dM2semimensural",
    "noteheads.dM3blackmensural",
    "noteheads.dM3mensural",
    "noteheads.dM3neomensural",
    "noteheads.dM3semimensural",
    "noteheads.drM2mensural",
    "noteheads.drM2neomensural",
    "noteheads.drM2semimensural",
    "noteheads.drM3mensural",
    "noteheads.drM3neomensural",
    "noteheads.drM3semimensural",
    "noteheads.s0",
    "noteheads.s0blackmensural",
    "noteheads.s0blackpetrucci",
    "noteheads.s0cross",
    "noteheads.s0diamond",
    "noteheads.s0do",
    "noteheads.s0doThin",
    "noteheads.s0doWalker",
    "noteheads.s0faWalker",
    "noteheads.s0harmonic",
    "noteheads.s0kievan",
    "noteheads.s0la",
    "noteheads.s0laFunk",
    "noteheads.s0laThin",
    "noteheads.s0laWalker",
    "noteheads.s0mensural",
    "noteheads.s0mi",
    "noteheads.s0miMirror",
    "noteheads.s0miThin",
    "noteheads.s0miWalker",
    "noteheads.s0neomensural",
    "noteheads.s0petrucci",
    "noteheads.s0re",
    "noteheads.s0reThin",
    "noteheads.s0reWalker",
    "noteheads.s0slash",
    "noteheads.s0sol",
    "noteheads.s0solFunk",
    "noteheads.s0ti",
    "noteheads.s0tiThin",
    "noteheads.s0tiWalker",
    "noteheads.s0triangle",
    "noteheads.s1",
    "noteheads.s1blackpetrucci",
    "noteheads.s1cross",
    "noteheads.s1diamond",
    "noteheads.s1kievan",
    "noteheads.s1la",
    "noteheads.s1laFunk",
    "noteheads.s1laThin",
    "noteheads.s1laWalker",
    "noteheads.s1mensural",
    "noteheads.s1mi",
    "noteheads.s1miMirror",
    "noteheads.s1miThin",
    "noteheads.s1miWalker",
    "noteheads.s1neomensural",
    "noteheads.s1petrucci",
    "noteheads.s1slash",
    "noteheads.s1sol",
    "noteheads.s1solFunk",
    "noteheads.s2",
    "noteheads.s2blackpetrucci",
    "noteheads.s2cross",
    "noteheads.s2diamond",
    "noteheads.s2harmonic",
    "noteheads.s2la",
    "noteheads.s2laFunk",
    "noteheads.s2laThin",
    "noteheads.s2laWalker",
    "noteheads.s2mensural",
    "noteheads.s2mi",
    "noteheads.s2miFunk",
    "noteheads.s2miMirror",
    "noteheads.s2miThin",
    "noteheads.s2miWalker",
    "noteheads.s2neomensural",
    "noteheads.s2petrucci",
    "noteheads.s2slash",
    "noteheads.s2sol",
    "noteheads.s2solFunk",
    "noteheads.s2xcircle",
    "noteheads.sM1",
    "noteheads.sM1blackmensural",
    "noteheads.sM1double",
    "noteheads.sM1kievan",
    "noteheads.sM1mensural",
    "noteheads.sM1neomensural",
    "noteheads.sM1semimensural",
    "noteheads.sM2blackligmensural",
    "noteheads.sM2kievan",
    "noteheads.sM2ligmensural",
    "noteheads.sM2semiligmensural",
    "noteheads.sM3blackligmensural",
    "noteheads.sM3ligmensural",
    "noteheads.sM3semiligmensural",
    "noteheads.shufnagel.lpes",
    "noteheads.shufnagel.punctum",
    "noteheads.shufnagel.virga",
    "noteheads.smedicaea.inclinatum",
    "noteheads.smedicaea.punctum",
    "noteheads.smedicaea.rvirga",
    "noteheads.smedicaea.virga",
    "noteheads.sr1kievan",
    "noteheads.srM1mensural",
    "noteheads.srM1neomensural",
    "noteheads.srM1semimensural",
    "noteheads.srM2ligmensural",
    "noteheads.srM2semiligmensural",
    "noteheads.srM3ligmensural",
    "noteheads.srM3semiligmensural",
    "noteheads.ssolesmes.auct.asc",
    "noteheads.ssolesmes.auct.desc",
    "noteheads.ssolesmes.incl.auctum",
    "noteheads.ssolesmes.incl.parvum",
    "noteheads.ssolesmes.oriscus",
    "noteheads.ssolesmes.stropha",
    "noteheads.ssolesmes.stropha.aucta",
    "noteheads.svaticana.cephalicus",
    "noteheads.svaticana.epiphonus",
    "noteheads.svaticana.inclinatum",
    "noteheads.svaticana.inner.cephalicus",
    "noteheads.svaticana.linea.punctum",
    "noteheads.svaticana.linea.punctum.cavum",
    "noteheads.svaticana.lpes",
    "noteheads.svaticana.plica",
    "noteheads.svaticana.punctum",
    "noteheads.svaticana.punctum.cavum",
    "noteheads.svaticana.quilisma",
    "noteheads.svaticana.reverse.plica",
    "noteheads.svaticana.reverse.vplica",
    "noteheads.svaticana.upes",
    "noteheads.svaticana.vepiphonus",
    "noteheads.svaticana.vlpes",
    "noteheads.svaticana.vplica",
    "noteheads.svaticana.vupes",
    "noteheads.u0doFunk",
    "noteheads.u0fa",
    "noteheads.u0faFunk",
    "noteheads.u0faThin",
    "noteheads.u0miFunk",
    "noteheads.u0reFunk",
    "noteheads.u0tiFunk",
    "noteheads.u1do",
    "noteheads.u1doFunk",
    "noteheads.u1doThin",
    "noteheads.u1doWalker",
    "noteheads.u1fa",
    "noteheads.u1faFunk",
    "noteheads.u1faThin",
    "noteheads.u1faWalker",
    "noteheads.u1miFunk",
    "noteheads.u1re",
    "noteheads.u1reFunk",
    "noteheads.u1reThin",
    "noteheads.u1reWalker",
    "noteheads.u1ti",
    "noteheads.u1tiFunk",
    "noteheads.u1tiThin",
    "noteheads.u1tiWalker",
    "noteheads.u1triangle",
    "noteheads.u2do",
    "noteheads.u2doFunk",
    "noteheads.u2doThin",
    "noteheads.u2doWalker",
    "noteheads.u2fa",
    "noteheads.u2faFunk",
    "noteheads.u2faThin",
    "noteheads.u2faWalker",
    "noteheads.u2kievan",
    "noteheads.u2re",
    "noteheads.u2reFunk",
    "noteheads.u2reThin",
    "noteheads.u2reWalker",
    "noteheads.u2ti",
    "noteheads.u2tiFunk",
    "noteheads.u2tiThin",
    "noteheads.u2tiWalker",
    "noteheads.u2triangle",
    "noteheads.u3kievan",
    "noteheads.uM2",
    "noteheads.uM2blackmensural",
    "noteheads.uM2mensural",
    "noteheads.uM2neomensural",
    "noteheads.uM2semimensural",
    "noteheads.uM3blackmensural",
    "noteheads.uM3mensural",
    "noteheads.uM3neomensural",
    "noteheads.uM3semimensural",
    "noteheads.urM2mensural",
    "noteheads.urM2neomensural",
    "noteheads.urM2semimensural",
    "noteheads.urM3mensural",
    "noteheads.urM3neomensural",
    "noteheads.urM3semimensural",
    "one",
    "p",
    "pedal.*",
    "pedal..",
    "pedal.M",
    "pedal.P",
    "pedal.Ped",
    "pedal.d",
    "pedal.e",
    "period",
    "plus",
    "r",
    "rests.0",
    "rests.0mensural",
    "rests.0neomensural",
    "rests.0o",
    "rests.1",
    "rests.1mensural",
    "rests.1neomensural",
    "rests.1o",
    "rests.2",
    "rests.2classical",
    "rests.2mensural",
    "rests.2neomensural",
    "rests.2z",
    "rests.3",
    "rests.3mensural",
    "rests.3neomensural",
    "rests.4",
    "rests.4mensural",
    "rests.4neomensural",
    "rests.5",
    "rests.6",
    "rests.7",
    "rests.M1",
    "rests.M1mensural",
    "rests.M1neomensural",
    "rests.M1o",
    "rests.M2",
    "rests.M2mensural",
    "rests.M2neomensural",
    "rests.M3",
    "rests.M3mensural",
    "rests.M3neomensural",
    "s",
    "scripts.arpeggio",
    "scripts.arpeggio.arrow.1",
    "scripts.arpeggio.arrow.M1",
    "scripts.augmentum",
    "scripts.barline.kievan",
    "scripts.caesura.curved",
    "scripts.caesura.straight",
    "scripts.circulus",
    "scripts.coda",
    "scripts.daccentus",
    "scripts.dfermata",
    "scripts.dlongfermata",
    "scripts.dmarcato",
    "scripts.downbow",
    "scripts.downmordent",
    "scripts.downprall",
    "scripts.dpedalheel",
    "scripts.dpedaltoe",
    "scripts.dportato",
    "scripts.dsemicirculus",
    "scripts.dshortfermata",
    "scripts.dsignumcongruentiae",
    "scripts.dstaccatissimo",
    "scripts.dverylongfermata",
    "scripts.espr",
    "scripts.flageolet",
    "scripts.halfopen",
    "scripts.halfopenvertical",
    "scripts.ictus",
    "scripts.lcomma",
    "scripts.lineprall",
    "scripts.lvarcomma",
    "scripts.mordent",
    "scripts.open",
    "scripts.prall",
    "scripts.pralldown",
    "scripts.prallmordent",
    "scripts.prallprall",
    "scripts.prallup",
    "scripts.rcomma",
    "scripts.reverseturn",
    "scripts.rvarcomma",
    "scripts.segno",
    "scripts.sforzato",
    "scripts.snappizzicato",
    "scripts.staccato",
    "scripts.stopped",
    "scripts.tenuto",
    "scripts.thumb",
    "scripts.tickmark",
    "scripts.trilelement",
    "scripts.trill",
    "scripts.trill_element",
    "scripts.turn",
    "scripts.uaccentus",
    "scripts.ufermata",
    "scripts.ulongfermata",
    "scripts.umarcato",
    "scripts.upbow",
    "scripts.upedalheel",
    "scripts.upedaltoe",
    "scripts.upmordent",
    "scripts.uportato",
    "scripts.upprall",
    "scripts.usemicirculus",
    "scripts.ushortfermata",
    "scripts.usignumcongruentiae",
    "scripts.ustaccatissimo",
    "scripts.uverylongfermata",
    "scripts.varcoda",
    "scripts.varsegno",
    "seven",
    "six",
    "space",
    "three",
    "ties.lyric.default",
    "ties.lyric.short",
    "timesig.C22",
    "timesig.C44",
    "timesig.mensural22",
    "timesig.mensural24",
    "timesig.mensural32",
    "timesig.mensural34",
    "timesig.mensural44",
    "timesig.mensural48",
    "timesig.mensural64",
    "timesig.mensural68",
    "timesig.mensural68alt",
    "timesig.mensural94",
    "timesig.mensural98",
    "timesig.neomensural22",
    "timesig.neomensural24",
    "timesig.neomensural32",
    "timesig.neomensural34",
    "timesig.neomensural44",
    "timesig.neomensural48",
    "timesig.neomensural64",
    "timesig.neomensural68",
    "timesig.neomensural68alt",
    "timesig.neomensural94",
    "timesig.neomensural98",
    "two",
    "z",
    "zero",
    ])



import abc
from abjad.system.AbjadValueObject import AbjadValueObject


class TimeRelation(AbjadValueObject):
    """
    Time relation.

    Time relations are immutable.
    """

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Time relations'

    __slots__ = (
        '_inequality',
        )

    ### INITIALIZER ###

    def __init__(self, inequality=None):
        from abjad import timespans
        if not inequality:
            inequality = timespans.CompoundInequality([
                'timespan_1.start_offset < timespan_2.start_offset',
                ])
        assert isinstance(
            inequality, timespans.CompoundInequality), repr(inequality)
        self._inequality = inequality

    ### SPECIAL METHODS ###

    @abc.abstractmethod
    def __call__(self):
        """
        Evaluates time relation.

        Returns true or false.
        """
        pass

    def __format__(self, format_specification=''):
        """
        Formats time relation.

        Returns string.
        """
        import abjad
        if format_specification in ('', 'storage'):
            return abjad.StorageFormatManager(self).get_storage_format()
        return str(self)

    def __hash__(self):
        """
        Hashes time relation.

        Required to be explicitly redefined on Python 3 if __eq__ changes.

        Returns integer.
        """
        return super().__hash__()

    ### PUBLIC PROPERTIES ###

    @property
    def inequality(self):
        """
        Time relation inequality.

        Return ineqality.
        """
        return self._inequality

    @abc.abstractproperty
    def is_fully_loaded(self):
        """
        Is true when both time relation terms are not none.

        Returns true or false.
        """
        pass

    @abc.abstractproperty
    def is_fully_unloaded(self):
        """
        Is true when both time relation terms are none.

        Returns true or false.
        """
        pass



"""
Instrument classes.
"""

import copy
import typing
from abjad.system.AbjadValueObject import AbjadValueObject
from abjad.pitch.NamedPitch import NamedPitch
from abjad.pitch.NamedPitchClass import NamedPitchClass
from abjad.pitch.PitchRange import PitchRange
from abjad.pitch.PitchSegment import PitchSegment
from abjad.utilities.Enumerator import Enumerator


class Instrument(AbjadValueObject):
    r"""
    Instrument.

    ..  container:: example

        Two instruments active on a single staff:

        >>> voice_1 = abjad.Voice("e'8 g'8 f'8 a'8")
        >>> flute = abjad.Flute()
        >>> abjad.attach(flute, voice_1[0], context='Voice')
        >>> flute_markup = abjad.Markup('(flute)', direction=abjad.Up)
        >>> abjad.attach(flute_markup, voice_1[0])
        >>> abjad.attach(abjad.LilyPondLiteral(r'\voiceOne'), voice_1)
        >>> voice_2 = abjad.Voice("c'2")
        >>> abjad.attach(abjad.LilyPondLiteral(r'\voiceTwo'), voice_2)
        >>> viola = abjad.Viola()
        >>> abjad.attach(viola, voice_2[0], context='Voice')
        >>> viola_markup = abjad.Markup('(viola)', direction=abjad.Down)
        >>> abjad.attach(viola_markup, voice_2[0])
        >>> staff = abjad.Staff([voice_1, voice_2], is_simultaneous=True)
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            <<
                \new Voice
                {
                    \voiceOne
                    e'8
                    ^ \markup { (flute) }
                    g'8
                    f'8
                    a'8
                }
                \new Voice
                {
                    \voiceTwo
                    c'2
                    _ \markup { (viola) }
                }
            >>

        >>> for leaf in abjad.select(voice_1).leaves():
        ...     leaf, abjad.inspect(leaf).effective(abjad.Instrument)
        ...
        (Note("e'8"), Flute())
        (Note("g'8"), Flute())
        (Note("f'8"), Flute())
        (Note("a'8"), Flute())

        >>> for leaf in abjad.select(voice_2).leaves():
        ...     leaf, abjad.inspect(leaf).effective(abjad.Instrument)
        ...
        (Note("c'2"), Viola())

    """

    ### CLASS VARIABLES ###

    __slots__ = (
        '_allowable_clefs',
        '_context',
        '_middle_c_sounding_pitch',
        '_name',
        '_name_markup',
        '_primary',
        '_middle_c_sounding_pitch',
        '_performer_names',
        '_pitch_range',
        '_short_name',
        '_short_name_markup',
        '_starting_clefs',
        )

    _format_slot = 'opening'

    _latent = True

    _parameter = 'INSTRUMENT'

    _publish_storage_format = True

    _redraw = True

    ### INITIALIZER ###

    def __init__(
        self,
        *,
        allowable_clefs=None,
        context=None,
        markup=None,
        middle_c_sounding_pitch=None,
        name=None,
        pitch_range=None,
        primary=None,
        short_name=None,
        short_markup=None,
        ):
        import abjad
        self._context = context or 'Staff'
        if name is not None:
            name = str(name)
        self._name = name
        if markup is not None:
            markup = abjad.Markup(markup)
        self._name_markup = markup
        if short_name is not None:
            short_name = str(short_name)
        self._short_name = short_name
        if short_markup is not None:
            short_markup = abjad.Markup(short_markup)
        self._short_name_markup = short_markup
        allowable_clefs = allowable_clefs or ('treble',)
        self._allowable_clefs = allowable_clefs
        if isinstance(pitch_range, str):
            pitch_range = abjad.PitchRange(pitch_range)
        elif isinstance(pitch_range, abjad.PitchRange):
            pitch_range = copy.copy(pitch_range)
        elif pitch_range is None:
            pitch_range = abjad.PitchRange()
        else:
            raise TypeError(pitch_range)
        self._pitch_range = pitch_range
        middle_c_sounding_pitch = (middle_c_sounding_pitch or
            abjad.NamedPitch("c'"))
        middle_c_sounding_pitch = abjad.NamedPitch(
            middle_c_sounding_pitch)
        self._middle_c_sounding_pitch = middle_c_sounding_pitch
        if primary is not None:
            primary = bool(primary)
        self._primary = primary
        self._performer_names = ['instrumentalist']
        self._starting_clefs = copy.copy(allowable_clefs)

    ### PRIVATE PROPERTIES ###

    @property
    def _lilypond_type(self):
        if isinstance(self.context, type):
            return self.context.__name__
        elif isinstance(self.context, str):
            return self.context
        else:
            return type(self.context).__name__

    ### PRIVATE METHODS ###

    def _attachment_test_all(self, component_expression):
        import abjad
        if abjad.inspect(component_expression).has_indicator(Instrument):
            return False
        return True

    def _get_format_specification(self):
        import abjad
        keywords = []
        return abjad.FormatSpecification(
            self,
            repr_args_values=[],
            repr_is_indented=False,
            repr_kwargs_names=keywords,
            )

    def _get_lilypond_format(self, context=None):
        return []

    def _initialize_default_name_markups(self):
        import abjad
        if self._name_markup is None:
            if self.name:
                string = self.name
                string = abjad.String(string).capitalize_start()
                markup = abjad.Markup(contents=string)
                self._name_markup = markup
            else:
                self._name_markup = None
        if self._short_name_markup is None:
            if self.short_name:
                string = self.short_name
                string = abjad.String(string).capitalize_start()
                markup = abjad.Markup(contents=string)
                self._short_name_markup = markup
            else:
                self._short_name_markup = None

    ### PUBLIC PROPERTIES ###

    @property
    def allowable_clefs(self):
        """
        Gets allowable clefs.

        Returns clef list.
        """
        if self._allowable_clefs is None:
            self._allowable_clefs = ('treble',)
        return self._allowable_clefs

    @property
    def context(self):
        """
        Gets (historically conventional) context of instrument.

        Defaults to ``'Staff'``.

        Returns lilypond type of context.

        Override with ``abjad.attach(..., context='...')``.
        """
        return self._context

    @property
    def latent(self):
        """
        Is true for all instruments.

        Class constant.

        Returns true.
        """
        return self._latent

    @property
    def markup(self):
        """
        Gets instrument name markup.

        Returns markup.
        """
        import abjad
        if self._name_markup is None:
            self._initialize_default_name_markups()
        if not isinstance(self._name_markup, abjad.Markup):
            markup = abjad.Markup(contents=self._name_markup)
            self._name_markup = markup
        if self._name_markup.contents != ('',):
            return self._name_markup

    @property
    def middle_c_sounding_pitch(self):
        """
        Gets sounding pitch of written middle C.

        Returns named pitch.
        """
        return self._middle_c_sounding_pitch

    @property
    def name(self):
        """
        Gets instrument name.

        Returns string.
        """
        return self._name

    @property
    def parameter(self) -> str:
        """
        Is set to ``'INSTRUMENT'`` for all instruments.

        Class constant.
        """
        return self._parameter

    @property
    def pitch_range(self):
        """
        Gets pitch range.

        Returns pitch range.
        """
        return self._pitch_range
        
    @property
    def primary(self):
        """
        Is true when instrument is historically conventional primary instrument
        (eg, flute) rather than doubling (eg, piccolo).
        """
        return self._primary

    @property
    def redraw(self):
        """
        Is true for all instruments.

        Class constant.

        Returns true.
        """
        return self._redraw

    @property
    def short_markup(self):
        """
        Gets short instrument name markup.

        Returns markup.
        """
        import abjad
        if self._short_name_markup is None:
            self._initialize_default_name_markups()
        if not isinstance(
            self._short_name_markup, abjad.Markup):
            markup = abjad.Markup(contents=self._short_name_markup)
            self._short_name_markup = markup
        if self._short_name_markup.contents != ('',):
            return self._short_name_markup

    @property
    def short_name(self):
        """
        Gets short instrument name.

        Returns string.
        """
        return self._short_name

    ### PUBLIC METHODS ###

    @staticmethod
    def transpose_from_sounding_pitch(argument):
        r"""
        Transpose notes and chords in `argument` from sounding pitch
        to written pitch:

        ..  container:: example

            >>> staff = abjad.Staff("<c' e' g'>4 d'4 r4 e'4")
            >>> clarinet = abjad.ClarinetInBFlat()
            >>> abjad.attach(clarinet, staff[0])
            >>> abjad.show(staff) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(staff)
                \new Staff
                {
                    <c' e' g'>4
                    d'4
                    r4
                    e'4
                }

            >>> abjad.Instrument.transpose_from_sounding_pitch(staff)
            >>> abjad.show(staff) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(staff)
                \new Staff
                {
                    <d' fs' a'>4
                    e'4
                    r4
                    fs'4
                }

        Returns none.
        """
        import abjad
        for leaf in abjad.iterate(argument).leaves(pitched=True):
            instrument = abjad.inspect(leaf).effective(abjad.Instrument)
            if not instrument:
                continue
            sounding_pitch = instrument.middle_c_sounding_pitch
            interval = abjad.NamedPitch('C4') - sounding_pitch
            interval *= -1
            if isinstance(leaf, abjad.Note):
                pitch = leaf.written_pitch
                pitch = interval.transpose(pitch)
                leaf.written_pitch = pitch
            elif isinstance(leaf, abjad.Chord):
                pitches = [
                    interval.transpose(pitch)
                    for pitch in leaf.written_pitches
                    ]
                leaf.written_pitches = pitches

    @staticmethod
    def transpose_from_written_pitch(argument):
        r"""
        Transposes notes and chords in `argument` from sounding pitch
        to written pitch.

        ..  container:: example

            >>> staff = abjad.Staff("<c' e' g'>4 d'4 r4 e'4")
            >>> clarinet = abjad.ClarinetInBFlat()
            >>> abjad.attach(clarinet, staff[0])
            >>> abjad.show(staff) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(staff)
                \new Staff
                {
                    <c' e' g'>4
                    d'4
                    r4
                    e'4
                }

            >>> abjad.Instrument.transpose_from_written_pitch(staff)
            >>> abjad.show(staff) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(staff)
                \new Staff
                {
                    <bf d' f'>4
                    c'4
                    r4
                    d'4
                }

        Returns none.
        """
        import abjad
        for leaf in abjad.iterate(argument).leaves(pitched=True):
            instrument = abjad.inspect(leaf).effective(abjad.Instrument)
            if not instrument:
                continue
            sounding_pitch = instrument.middle_c_sounding_pitch
            interval = abjad.NamedPitch('C4') - sounding_pitch
            if isinstance(leaf, abjad.Note):
                written_pitch = leaf.written_pitch
                written_pitch = interval.transpose(written_pitch)
                leaf.written_pitch = written_pitch
            elif isinstance(leaf, abjad.Chord):
                pitches = [
                    interval.transpose(pitch)
                    for pitch in leaf.written_pitches
                    ]
                leaf.written_pitches = pitches


class StringNumber(AbjadValueObject):
    """
    String number.

    ..  container:: example

        String I:

        >>> indicator = abjad.StringNumber(1)
        >>> abjad.f(indicator)
        abjad.StringNumber(
            numbers=(1,),
            )

    ..  container:: example

        Strings II and III:

        >>> indicator = abjad.StringNumber((2, 3))
        >>> abjad.f(indicator)
        abjad.StringNumber(
            numbers=(2, 3),
            )

    """

    ### CLASS VARIABLES

    __slots__ = (
        '_numbers',
        )

    _publish_storage_format = True

    ### INITIALIZER ###

    def __init__(
        self,
        numbers: typing.Union[int, typing.Iterable[int]] = None,
        ) -> None:
        if numbers is None:
            numbers_: typing.Tuple[int, ...] = ()
        elif isinstance(numbers, int):
            numbers_ = (numbers,)
        else:
            numbers_ = tuple(numbers)
        assert isinstance(numbers_, tuple), repr(numbers_)
        numbers_ = tuple(int(_) for _ in numbers_)
        assert all(0 < _ < 7 for _ in numbers_)
        self._numbers = numbers_

    ### PUBLIC PROPERTIES ###

    @property
    def numbers(self) -> typing.Tuple[int, ...]:
        """
        Gets numbers.

        ..  container:: example

            String I:

            >>> indicator = abjad.StringNumber(1)
            >>> indicator.numbers
            (1,)

            >>> indicator = abjad.StringNumber((2, 3))
            >>> indicator.numbers
            (2, 3)

        """
        return self._numbers

    @property
    def roman_numerals(self) -> typing.Tuple[str, ...]:
        """
        Gets roman numerals of string number indicator.

        ..  container:: example

            String I:

            >>> indicator = abjad.StringNumber(1)
            >>> indicator.roman_numerals
            ('i',)

        ..  container:: example

            Strings II and III:

            >>> indicator = abjad.StringNumber((2, 3))
            >>> indicator.roman_numerals
            ('ii', 'iii')

        """
        numerals = ('i', 'ii', 'iii', 'iv', 'v', 'vi')
        result = []
        for number in self.numbers:
            numeral = numerals[number - 1]
            result.append(numeral)
        return tuple(result)

    @property
    def tweaks(self) -> None:
        """
        Are not implemented on string number.
        """
        pass


class Tuning(AbjadValueObject):
    """
    Tuning.

    ..  container:: example

        Violin tuning:

        >>> indicator = abjad.Tuning(pitches=('G3', 'D4', 'A4', 'E5'))
        >>> abjad.f(indicator)
        abjad.Tuning(
            pitches=abjad.PitchSegment(
                (
                    abjad.NamedPitch('g'),
                    abjad.NamedPitch("d'"),
                    abjad.NamedPitch("a'"),
                    abjad.NamedPitch("e''"),
                    ),
                item_class=abjad.NamedPitch,
                ),
            )

    """

    ### CLASS VARIABLES ###

    __slots__ = (
        '_pitches',
        )

    _publish_storage_format = True

    ### INITIALIZER ###

    def __init__(
        self,
        pitches: typing.Union['Tuning', typing.Iterable] = None,
        ) -> None:
        if pitches is not None:
            if isinstance(pitches, type(self)):
                pitches = pitches.pitches
            pitches = PitchSegment(
                items=pitches,
                item_class=NamedPitch,
                )
        self._pitches: typing.Optional[PitchSegment] = pitches

    ### PUBLIC PROPERTIES ###

    @property
    def pitch_ranges(self) -> typing.List[PitchRange]:
        """
        Gets two-octave pitch-ranges for each pitch in this tuning.

        ..  container:: example

            >>> indicator = abjad.Tuning(pitches=('G3', 'D4', 'A4', 'E5'))
            >>> for range_ in indicator.pitch_ranges:
            ...     range_
            PitchRange('[G3, G5]')
            PitchRange('[D4, D6]')
            PitchRange('[A4, A6]')
            PitchRange('[E5, E7]')

        """
        result = []
        for pitch in self.pitches or []:
            pitch_range = PitchRange.from_pitches(pitch, pitch + 24)
            result.append(pitch_range)
        return result

    @property
    def pitches(self) -> typing.Optional[PitchSegment]:
        """
        Gets pitches of tuning.

        ..  container:: example

            >>> indicator = abjad.Tuning(pitches=('G3', 'D4', 'A4', 'E5'))
            >>> pitches = indicator.pitches
            >>> abjad.f(pitches)
            abjad.PitchSegment(
                (
                    abjad.NamedPitch('g'),
                    abjad.NamedPitch("d'"),
                    abjad.NamedPitch("a'"),
                    abjad.NamedPitch("e''"),
                    ),
                item_class=abjad.NamedPitch,
                )

        """
        return self._pitches

    @property
    def tweaks(self) -> None:
        """
        Are not implemented on tuning.
        """
        pass

    ### PUBLIC METHODS ###

    def get_pitch_ranges_by_string_number(
        self,
        string_number: StringNumber,
        ) -> typing.Tuple[PitchRange, ...]:
        """
        Gets tuning pitch ranges by string number.

        ..  container:: example

            Violin tuning:

            >>> tuning = abjad.Tuning(('G3', 'D4', 'A4', 'E5'))
            >>> string_number = abjad.StringNumber((2, 3))
            >>> tuning.get_pitch_ranges_by_string_number(string_number)
            (PitchRange('[A4, A6]'), PitchRange('[D4, D6]'))

        """
        if not isinstance(string_number, StringNumber):
            string_number = StringNumber(string_number)
        assert isinstance(string_number, StringNumber)
        pitch_ranges = self.pitch_ranges
        result = []
        for number in string_number.numbers:
            index = -number
            pitch_range = pitch_ranges[index]
            result.append(pitch_range)
        return tuple(result)

    def get_pitches_by_string_number(
        self,
        string_number: StringNumber,
        ) -> typing.Tuple[NamedPitch, ...]:
        """
        Gets tuning pitches by string number.

        ..  container:: example

            Violin tuning:

            >>> tuning = abjad.Tuning(('G3', 'D4', 'A4', 'E5'))
            >>> string_number = abjad.StringNumber((2, 3))
            >>> tuning.get_pitches_by_string_number(string_number)
            (NamedPitch("a'"), NamedPitch("d'"))

        """
        if not isinstance(string_number, StringNumber):
            string_number = StringNumber(string_number)
        assert isinstance(string_number, StringNumber)
        assert self.pitches is not None
        result = []
        for number in string_number.numbers:
            index = -number
            pitch = self.pitches[index]
            result.append(pitch)
        return tuple(result)

    def voice_pitch_classes(
        self,
        pitch_classes,
        allow_open_strings: bool = True,
        ):
        r"""
        Voices ``pitch_classes``.

        ..  container:: example

            >>> tuning = abjad.Tuning(('G3', 'D4', 'A4', 'E5'))
            >>> voicings = tuning.voice_pitch_classes(('a',))
            >>> for voicing in voicings:
            ...     voicing
            ...
            (NamedPitch('a'), None, None, None)
            (None, None, None, NamedPitch("a''"))
            (None, None, None, NamedPitch("a'''"))
            (None, None, NamedPitch("a'"), None)
            (None, None, NamedPitch("a''"), None)
            (None, None, NamedPitch("a'''"), None)
            (None, NamedPitch("a'"), None, None)
            (None, NamedPitch("a''"), None, None)
            (NamedPitch("a'"), None, None, None)

            >>> voicings = tuning.voice_pitch_classes(
            ...     ('a', 'd'),
            ...     allow_open_strings=False,
            ...     )
            >>> for voicing in voicings:
            ...     voicing
            ...
            (NamedPitch('a'), None, None, NamedPitch("d'''"))
            (NamedPitch('a'), None, None, NamedPitch("d''''"))
            (NamedPitch('a'), None, NamedPitch("d''"), None)
            (NamedPitch('a'), None, NamedPitch("d'''"), None)
            (NamedPitch('a'), NamedPitch("d''"), None, None)
            (NamedPitch('a'), NamedPitch("d'''"), None, None)
            (None, None, NamedPitch("d''"), NamedPitch("a''"))
            (None, None, NamedPitch("d''"), NamedPitch("a'''"))
            (None, None, NamedPitch("a''"), NamedPitch("d'''"))
            (None, None, NamedPitch("a''"), NamedPitch("d''''"))
            (None, None, NamedPitch("d'''"), NamedPitch("a''"))
            (None, None, NamedPitch("d'''"), NamedPitch("a'''"))
            (None, None, NamedPitch("a'''"), NamedPitch("d'''"))
            (None, None, NamedPitch("a'''"), NamedPitch("d''''"))
            (None, NamedPitch("a'"), None, NamedPitch("d'''"))
            (None, NamedPitch("a'"), None, NamedPitch("d''''"))
            (None, NamedPitch("a'"), NamedPitch("d''"), None)
            (None, NamedPitch("a'"), NamedPitch("d'''"), None)
            (None, NamedPitch("d''"), None, NamedPitch("a''"))
            (None, NamedPitch("d''"), None, NamedPitch("a'''"))
            (None, NamedPitch("d''"), NamedPitch("a''"), None)
            (None, NamedPitch("d''"), NamedPitch("a'''"), None)
            (None, NamedPitch("a''"), None, NamedPitch("d'''"))
            (None, NamedPitch("a''"), None, NamedPitch("d''''"))
            (None, NamedPitch("a''"), NamedPitch("d''"), None)
            (None, NamedPitch("a''"), NamedPitch("d'''"), None)
            (None, NamedPitch("d'''"), None, NamedPitch("a''"))
            (None, NamedPitch("d'''"), None, NamedPitch("a'''"))
            (None, NamedPitch("d'''"), NamedPitch("a''"), None)
            (None, NamedPitch("d'''"), NamedPitch("a'''"), None)
            (NamedPitch("d'"), None, None, NamedPitch("a''"))
            (NamedPitch("d'"), None, None, NamedPitch("a'''"))
            (NamedPitch("d'"), None, NamedPitch("a''"), None)
            (NamedPitch("d'"), None, NamedPitch("a'''"), None)
            (NamedPitch("d'"), NamedPitch("a'"), None, None)
            (NamedPitch("d'"), NamedPitch("a''"), None, None)
            (NamedPitch("a'"), None, None, NamedPitch("d'''"))
            (NamedPitch("a'"), None, None, NamedPitch("d''''"))
            (NamedPitch("a'"), None, NamedPitch("d''"), None)
            (NamedPitch("a'"), None, NamedPitch("d'''"), None)
            (NamedPitch("a'"), NamedPitch("d''"), None, None)
            (NamedPitch("a'"), NamedPitch("d'''"), None, None)
            (NamedPitch("d''"), None, None, NamedPitch("a''"))
            (NamedPitch("d''"), None, None, NamedPitch("a'''"))
            (NamedPitch("d''"), None, NamedPitch("a''"), None)
            (NamedPitch("d''"), None, NamedPitch("a'''"), None)
            (NamedPitch("d''"), NamedPitch("a'"), None, None)
            (NamedPitch("d''"), NamedPitch("a''"), None, None)

        """
        assert self.pitches is not None
        pitch_classes = [NamedPitchClass(_) for _ in pitch_classes]
        pitch_classes.extend([None] * (len(self.pitches) - len(pitch_classes)))
        enumerator = Enumerator(pitch_classes)
        permutations = enumerator.yield_permutations()
        permutations = set([tuple(_) for _ in permutations])
        pitch_ranges = self.pitch_ranges
        result: typing.List[
            typing.Tuple[typing.Union[NamedPitch, None], ...]
            ] = []
        for permutation in permutations:
            sequences = []
            for pitch_range, pitch_class in zip(pitch_ranges, permutation):
                if pitch_class is None:
                    sequences.append([None])
                    continue
                pitches = pitch_range.voice_pitch_class(pitch_class)
                if not allow_open_strings:
                    pitches = [
                        pitch for pitch in pitches
                        if pitch != pitch_range.start_pitch
                        ]
                if not pitches:
                    pitches = [None]
                sequences.append(pitches)
            enumerator = Enumerator(sequences)
            subresult = enumerator.yield_outer_product()
            subresult = [tuple(x) for x in subresult]
            result.extend(subresult)
        result.sort()
        return tuple(result)


class Accordion(Instrument):
    r"""
    Accordion.

    ..  container:: example

        >>> staff_group = abjad.StaffGroup(lilypond_type='PianoStaff')
        >>> staff_group.append(abjad.Staff("c'4 d'4 e'4 f'4"))
        >>> staff_group.append(abjad.Staff("c'2 b2"))
        >>> accordion = abjad.Accordion()
        >>> abjad.attach(accordion, staff_group[0][0])
        >>> abjad.attach(abjad.Clef('bass'), staff_group[1][0])
        >>> abjad.show(staff_group) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff_group)
            \new PianoStaff
            <<
                \new Staff
                {
                    c'4
                    d'4
                    e'4
                    f'4
                }
                \new Staff
                {
                    \clef "bass"
                    c'2
                    b2
                }
            >>

    """

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(
        self,
        *,
        allowable_clefs=('treble', 'bass'),
        context='StaffGroup',
        name='accordion',
        markup=None,
        middle_c_sounding_pitch=None,
        pitch_range='[E1, C8]',
        primary=True,
        short_markup=None,
        short_name='acc.',
        ):
        Instrument.__init__(
            self,
            allowable_clefs=allowable_clefs,
            context=context,
            name=name,
            markup=markup,
            short_markup=short_markup,
            middle_c_sounding_pitch=middle_c_sounding_pitch,
            pitch_range=pitch_range,
            short_name=short_name,
            )


class AltoFlute(Instrument):
    r"""
    Alto flute.

    ..  container:: example

        >>> staff = abjad.Staff("c'4 d'4 e'4 fs'4")
        >>> alto_flute = abjad.AltoFlute()
        >>> abjad.attach(alto_flute, staff[0])
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                c'4
                d'4
                e'4
                fs'4
            }

    """

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(
        self,
        *,
        name='alto flute',
        short_name='alt. fl.',
        markup=None,
        short_markup=None,
        allowable_clefs=None,
        context=None,
        middle_c_sounding_pitch='G3',
        pitch_range='[G3, G6]',
        ):
        Instrument.__init__(
            self,
            name=name,
            short_name=short_name,
            markup=markup,
            short_markup=short_markup,
            allowable_clefs=allowable_clefs,
            context=context,
            middle_c_sounding_pitch=middle_c_sounding_pitch,
            pitch_range=pitch_range,
            )


class AltoSaxophone(Instrument):
    r"""
    Alto saxophone.

    ..  container:: example

        >>> staff = abjad.Staff("c'4 d'4 e'4 fs'4")
        >>> alto_saxophone = abjad.AltoSaxophone()
        >>> abjad.attach(alto_saxophone, staff[0])
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                c'4
                d'4
                e'4
                fs'4
            }

    """

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(
        self,
        *,
        name='alto saxophone',
        short_name='alt. sax.',
        markup=None,
        short_markup=None,
        allowable_clefs=None,
        context=None,
        middle_c_sounding_pitch='Eb3',
        pitch_range='[Db3, A5]',
        primary=True,
        ):
        Instrument.__init__(
            self,
            name=name,
            short_name=short_name,
            markup=markup,
            short_markup=short_markup,
            allowable_clefs=allowable_clefs,
            context=context,
            middle_c_sounding_pitch=middle_c_sounding_pitch,
            pitch_range=pitch_range,
            primary=primary,
            )


class AltoTrombone(Instrument):
    r"""
    Alto trombone.

    ..  container:: example

        >>> staff = abjad.Staff("c4 d4 e4 fs4")
        >>> clef = abjad.Clef('bass')
        >>> abjad.attach(clef, staff[0])
        >>> alto_trombone = abjad.AltoTrombone()
        >>> abjad.attach(alto_trombone, staff[0])
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                \clef "bass"
                c4
                d4
                e4
                fs4
            }

    """

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(
        self,
        *,
        name='alto trombone',
        short_name='alt. trb.',
        markup=None,
        short_markup=None,
        allowable_clefs=('bass', 'tenor'),
        context=None,
        middle_c_sounding_pitch=None,
        pitch_range='[A2, Bb5]',
        ):
        Instrument.__init__(
            self,
            name=name,
            short_name=short_name,
            markup=markup,
            short_markup=short_markup,
            allowable_clefs=allowable_clefs,
            context=context,
            middle_c_sounding_pitch=middle_c_sounding_pitch,
            pitch_range=pitch_range,
            )


class AltoVoice(Instrument):
    r"""
    Alto voice.

    ..  container:: example

        >>> staff = abjad.Staff("c'4 d'4 e'4 fs'4")
        >>> alto = abjad.AltoVoice()
        >>> abjad.attach(alto, staff[0])
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                c'4
                d'4
                e'4
                fs'4
            }

    """

    ### CLASS VARIABLES ###

    __slots__ = ()

    performer_abbreviation = 'alto'

    ### INITIALIZER ###

    def __init__(
        self,
        *,
        name='alto',
        short_name='alto',
        markup=None,
        short_markup=None,
        allowable_clefs=None,
        context=None,
        middle_c_sounding_pitch=None,
        pitch_range='[F3, G5]',
        ):
        Instrument.__init__(
            self,
            name=name,
            short_name=short_name,
            markup=markup,
            short_markup=short_markup,
            allowable_clefs=allowable_clefs,
            context=context,
            middle_c_sounding_pitch=middle_c_sounding_pitch,
            pitch_range=pitch_range,
            )


class BaritoneSaxophone(Instrument):
    r"""
    Baritone saxophone.

    ..  container:: example

        >>> staff = abjad.Staff("c'4 d'4 e'4 fs'4")
        >>> baritone_saxophone = abjad.BaritoneSaxophone()
        >>> abjad.attach(baritone_saxophone, staff[0])
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                c'4
                d'4
                e'4
                fs'4
            }

    """

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(
        self,
        *,
        name='baritone saxophone',
        short_name='bar. sax.',
        markup=None,
        short_markup=None,
        allowable_clefs=None,
        context=None,
        middle_c_sounding_pitch='Eb2',
        pitch_range='[C2, Ab4]',
        ):
        Instrument.__init__(
            self,
            name=name,
            short_name=short_name,
            markup=markup,
            short_markup=short_markup,
            allowable_clefs=allowable_clefs,
            context=context,
            middle_c_sounding_pitch=middle_c_sounding_pitch,
            pitch_range=pitch_range,
            )


class BaritoneVoice(Instrument):
    r"""
    Baritone voice.

    ..  container:: example

        >>> staff = abjad.Staff("c4 d4 e4 fs4")
        >>> baritone = abjad.BaritoneVoice()
        >>> abjad.attach(baritone, staff[0])
        >>> clef = abjad.Clef('bass')
        >>> abjad.attach(clef, staff[0])
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                \clef "bass"
                c4
                d4
                e4
                fs4
            }

    """

    ### CLASS VARIABLES ###

    __slots__ = ()

    performer_abbreviation = 'bar.'

    ### INITIALIZER ###

    def __init__(
        self,
        *,
        name='baritone',
        short_name='bar.',
        markup=None,
        short_markup=None,
        allowable_clefs=('bass',),
        context=None,
        middle_c_sounding_pitch=None,
        pitch_range='[A2, A4]',
        ):
        Instrument.__init__(
            self,
            name=name,
            short_name=short_name,
            markup=markup,
            short_markup=short_markup,
            allowable_clefs=allowable_clefs,
            context=context,
            middle_c_sounding_pitch=middle_c_sounding_pitch,
            pitch_range=pitch_range,
            )


class BassClarinet(Instrument):
    r"""
    Bass clarinet.

    ..  container:: example

        >>> staff = abjad.Staff("c'4 d'4 e'4 fs'4")
        >>> bass_clarinet = abjad.BassClarinet()
        >>> abjad.attach(bass_clarinet, staff[0])
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                c'4
                d'4
                e'4
                fs'4
            }

    """

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(
        self,
        *,
        name='bass clarinet',
        short_name='bass cl.',
        markup=None,
        short_markup=None,
        allowable_clefs=('treble', 'bass'),
        context=None,
        middle_c_sounding_pitch='Bb2',
        pitch_range='[Bb1, G5]',
        ):
        Instrument.__init__(
            self,
            name=name,
            short_name=short_name,
            markup=markup,
            short_markup=short_markup,
            allowable_clefs=allowable_clefs,
            context=context,
            middle_c_sounding_pitch=middle_c_sounding_pitch,
            pitch_range=pitch_range,
            )


class BassFlute(Instrument):
    r"""
    Bass flute.

    ..  container:: example

        >>> staff = abjad.Staff("c'4 d'4 e'4 fs'4")
        >>> bass_flute = abjad.BassFlute()
        >>> abjad.attach(bass_flute, staff[0])
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                c'4
                d'4
                e'4
                fs'4
            }

    """

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(
        self,
        *,
        name='bass flute',
        short_name='bass fl.',
        markup=None,
        short_markup=None,
        allowable_clefs=None,
        context=None,
        middle_c_sounding_pitch='C3',
        pitch_range='[C3, C6]',
        ):
        Instrument.__init__(
            self,
            name=name,
            short_name=short_name,
            markup=markup,
            short_markup=short_markup,
            allowable_clefs=allowable_clefs,
            context=context,
            middle_c_sounding_pitch=middle_c_sounding_pitch,
            pitch_range=pitch_range,
            )


class BassSaxophone(Instrument):
    r"""
    Bass saxophone.

    ..  container:: example

        >>> staff = abjad.Staff("c'4 d'4 e'4 fs'4")
        >>> bass_saxophone = abjad.BassSaxophone()
        >>> abjad.attach(bass_saxophone, staff[0])
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                c'4
                d'4
                e'4
                fs'4
            }

    """

    ### CLASS ATTRIBUTES ###

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(
        self,
        *,
        name='bass saxophone',
        short_name='bass sax.',
        markup=None,
        short_markup=None,
        allowable_clefs=None,
        context=None,
        middle_c_sounding_pitch='Bb1',
        pitch_range='[Ab2, E4]',
        ):
        Instrument.__init__(
            self,
            name=name,
            short_name=short_name,
            markup=markup,
            short_markup=short_markup,
            allowable_clefs=allowable_clefs,
            context=context,
            middle_c_sounding_pitch=middle_c_sounding_pitch,
            pitch_range=pitch_range,
            )


class BassTrombone(Instrument):
    r"""
    Bass trombone.

    ..  container:: example

        >>> staff = abjad.Staff("c'4 d'4 e'4 fs'4")
        >>> clef = abjad.Clef('bass')
        >>> abjad.attach(clef, staff[0])
        >>> bass_trombone = abjad.BassTrombone()
        >>> abjad.attach(bass_trombone, staff[0])
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                \clef "bass"
                c'4
                d'4
                e'4
                fs'4
            }

    """

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(
        self,
        *,
        name='bass trombone',
        short_name='bass trb.',
        markup=None,
        short_markup=None,
        allowable_clefs=('bass',),
        context=None,
        middle_c_sounding_pitch=None,
        pitch_range='[C2, F4]',
        ):
        Instrument.__init__(
            self,
            name=name,
            short_name=short_name,
            markup=markup,
            short_markup=short_markup,
            allowable_clefs=allowable_clefs,
            context=context,
            middle_c_sounding_pitch=middle_c_sounding_pitch,
            pitch_range=pitch_range,
            )


class BassVoice(Instrument):
    r"""
    Bass voice.

    ..  container:: example

        >>> staff = abjad.Staff("c4 d4 e4 fs4")
        >>> bass = abjad.BassVoice()
        >>> abjad.attach(bass, staff[0])
        >>> clef = abjad.Clef('bass')
        >>> abjad.attach(clef, staff[0])
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                \clef "bass"
                c4
                d4
                e4
                fs4
            }

    """

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(
        self,
        *,
        name='bass',
        short_name='bass',
        markup=None,
        short_markup=None,
        allowable_clefs=('bass',),
        context=None,
        middle_c_sounding_pitch=None,
        pitch_range='[E2, F4]',
        primary=True,
        ):
        Instrument.__init__(
            self,
            name=name,
            short_name=short_name,
            markup=markup,
            short_markup=short_markup,
            allowable_clefs=allowable_clefs,
            context=context,
            middle_c_sounding_pitch=middle_c_sounding_pitch,
            pitch_range=pitch_range,
            primary=primary,
            )


class Bassoon(Instrument):
    r"""
    Bassoon.

    ..  container:: example

        >>> staff = abjad.Staff("c'4 d'4 e'4 fs'4")
        >>> clef = abjad.Clef('bass')
        >>> abjad.attach(clef, staff[0])
        >>> bassoon = abjad.Bassoon()
        >>> abjad.attach(bassoon, staff[0])
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                \clef "bass"
                c'4
                d'4
                e'4
                fs'4
            }

    """

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(
        self,
        *,
        name='bassoon',
        short_name='bsn.',
        markup=None,
        short_markup=None,
        allowable_clefs=('bass', 'tenor'),
        context=None,
        middle_c_sounding_pitch=None,
        pitch_range='[Bb1, Eb5]',
        primary=True
        ):
        Instrument.__init__(
            self,
            name=name,
            short_name=short_name,
            markup=markup,
            short_markup=short_markup,
            allowable_clefs=allowable_clefs,
            context=context,
            middle_c_sounding_pitch=middle_c_sounding_pitch,
            pitch_range=pitch_range,
            primary=primary,
            )


class Cello(Instrument):
    r"""
    Cello.

    ..  container:: example

        >>> staff = abjad.Staff("c'4 d'4 e'4 fs'4")
        >>> clef = abjad.Clef('bass')
        >>> abjad.attach(clef, staff[0])
        >>> cello = abjad.Cello()
        >>> abjad.attach(cello, staff[0])
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                \clef "bass"
                c'4
                d'4
                e'4
                fs'4
            }

    """

    ### CLASS VARIABLES ###

    __slots__ = (
        '_default_tuning',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        *,
        name='cello',
        short_name='vc.',
        markup=None,
        short_markup=None,
        allowable_clefs=('bass', 'tenor', 'treble'),
        context=None,
        default_tuning=('C2', 'G2', 'D3', 'A3'),
        middle_c_sounding_pitch=None,
        pitch_range='[C2, G5]',
        primary=True,
        ):
        Instrument.__init__(
            self,
            name=name,
            short_name=short_name,
            markup=markup,
            short_markup=short_markup,
            allowable_clefs=allowable_clefs,
            context=context,
            middle_c_sounding_pitch=middle_c_sounding_pitch,
            pitch_range=pitch_range,
            primary=primary,
            )
        self._default_tuning = Tuning(default_tuning)

    ### PUBLIC PROPERTIES ###

    @property
    def default_tuning(self):
        """
        Gets cello's default tuning.

        ..  container:: example

            >>> cello = abjad.Cello()
            >>> cello.default_tuning
            Tuning(pitches=PitchSegment(['c,', 'g,', 'd', 'a']))

        Returns tuning.
        """
        return self._default_tuning


class ClarinetInA(Instrument):
    r"""
    Clarinet in A.

    ..  container:: example

        >>> staff = abjad.Staff("c'4 d'4 e'4 fs'4")
        >>> clarinet = abjad.ClarinetInA()
        >>> abjad.attach(clarinet, staff[0])
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                c'4
                d'4
                e'4
                fs'4
            }

    """

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(
        self,
        *,
        name='clarinet in A',
        short_name=r'cl. A \natural',
        markup=None,
        short_markup=None,
        allowable_clefs=None,
        context=None,
        middle_c_sounding_pitch='A3',
        pitch_range='[Db3, A6]',
        ):
        Instrument.__init__(
            self,
            name=name,
            short_name=short_name,
            markup=markup,
            short_markup=short_markup,
            allowable_clefs=allowable_clefs,
            context=context,
            middle_c_sounding_pitch=middle_c_sounding_pitch,
            pitch_range=pitch_range,
            )


class ClarinetInBFlat(Instrument):
    r"""
    Clarinet in B-flat.

    ..  container:: example

        >>> staff = abjad.Staff("c'4 d'4 e'4 fs'4")
        >>> clarinet = abjad.ClarinetInBFlat()
        >>> abjad.attach(clarinet, staff[0])
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                c'4
                d'4
                e'4
                fs'4
            }

    """

    ### CLASS VARIABLES ###

    __slots__ = ()

    performer_abbreviation = 'cl.'

    ### INITIALIZER ###

    def __init__(
        self,
        *,
        name='clarinet in B-flat',
        short_name='cl. in B-flat',
        markup=None,
        short_markup=None,
        allowable_clefs=None,
        context=None,
        middle_c_sounding_pitch='Bb3',
        pitch_range='[D3, Bb6]',
        primary=True,
        ):
        Instrument.__init__(
            self,
            name=name,
            short_name=short_name,
            markup=markup,
            short_markup=short_markup,
            allowable_clefs=allowable_clefs,
            context=context,
            middle_c_sounding_pitch=middle_c_sounding_pitch,
            pitch_range=pitch_range,
            primary=primary,
            )
            

class ClarinetInEFlat(Instrument):
    r"""
    Clarinet in E-flat.

    ..  container:: example

        >>> staff = abjad.Staff("c'4 d'4 e'4 fs'4")
        >>> clarinet = abjad.ClarinetInEFlat()
        >>> abjad.attach(clarinet, staff[0])
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                c'4
                d'4
                e'4
                fs'4
            }

    """

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(
        self,
        *,
        name='clarinet in E-flat',
        short_name='cl. E-flat',
        markup=None,
        short_markup=None,
        allowable_clefs=None,
        context=None,
        middle_c_sounding_pitch='Eb4',
        pitch_range='[F3, C7]',
        ):
        Instrument.__init__(
            self,
            name=name,
            short_name=short_name,
            markup=markup,
            short_markup=short_markup,
            allowable_clefs=allowable_clefs,
            context=context,
            middle_c_sounding_pitch=middle_c_sounding_pitch,
            pitch_range=pitch_range,
            )


class Contrabass(Instrument):
    r"""
    Contrabass.

    ..  container:: example

        >>> staff = abjad.Staff("c'4 d'4 e'4 fs'4")
        >>> clef = abjad.Clef('bass')
        >>> abjad.attach(clef, staff[0])
        >>> contrabass = abjad.Contrabass()
        >>> abjad.attach(contrabass, staff[0])
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                \clef "bass"
                c'4
                d'4
                e'4
                fs'4
            }

    """

    ### CLASS VARIABLES ###

    __slots__ = (
        '_default_tuning',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        *,
        name='contrabass',
        short_name='cb.',
        markup=None,
        short_markup=None,
        allowable_clefs=('bass', 'treble'),
        context=None,
        default_tuning=('C1', 'A1', 'D2', 'G2'),
        middle_c_sounding_pitch='C3',
        pitch_range='[C1, G4]',
        primary=True,
        ):
        Instrument.__init__(
            self,
            name=name,
            short_name=short_name,
            markup=markup,
            short_markup=short_markup,
            allowable_clefs=allowable_clefs,
            context=context,
            middle_c_sounding_pitch=middle_c_sounding_pitch,
            pitch_range=pitch_range,
            primary=primary,
            )
        self._default_tuning = Tuning(default_tuning)

    ### PUBLIC PROPERTIES ###

    @property
    def default_tuning(self):
        """
        Gets contrabass's default tuning.

        ..  container:: example

            >>> contrabass = abjad.Contrabass()
            >>> contrabass.default_tuning
            Tuning(pitches=PitchSegment(['c,,', 'a,,', 'd,', 'g,']))

        Returns tuning.
        """
        return self._default_tuning


class ContrabassClarinet(Instrument):
    r"""
    Contrassbass clarinet.

    ..  container:: example

        >>> staff = abjad.Staff("c'4 d'4 e'4 fs'4")
        >>> contrabass_clarinet = abjad.ContrabassClarinet()
        >>> abjad.attach(contrabass_clarinet, staff[0])
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                c'4
                d'4
                e'4
                fs'4
            }

    """

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(
        self,
        *,
        name='contrabass clarinet',
        short_name='cbass. cl.',
        markup=None,
        short_markup=None,
        allowable_clefs=('treble', 'bass'),
        context=None,
        middle_c_sounding_pitch='Bb1',
        pitch_range='[Bb0, G4]',
        ):
        Instrument.__init__(
            self,
            name=name,
            short_name=short_name,
            markup=markup,
            short_markup=short_markup,
            allowable_clefs=allowable_clefs,
            context=context,
            middle_c_sounding_pitch=middle_c_sounding_pitch,
            pitch_range=pitch_range,
            )


class ContrabassFlute(Instrument):
    r"""
    Contrabass flute.

    ..  container:: example

        >>> staff = abjad.Staff("c'4 d'4 e'4 fs'4")
        >>> contrabass_flute = abjad.ContrabassFlute()
        >>> abjad.attach(contrabass_flute, staff[0])
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                c'4
                d'4
                e'4
                fs'4
            }

    """

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(
        self,
        *,
        name='contrabass flute',
        short_name='cbass. fl.',
        markup=None,
        short_markup=None,
        allowable_clefs=None,
        context=None,
        middle_c_sounding_pitch='G2',
        pitch_range='[G2, G5]',
        ):
        Instrument.__init__(
            self,
            name=name,
            short_name=short_name,
            markup=markup,
            short_markup=short_markup,
            allowable_clefs=allowable_clefs,
            context=context,
            middle_c_sounding_pitch=middle_c_sounding_pitch,
            pitch_range=pitch_range,
            )


class ContrabassSaxophone(Instrument):
    r"""
    Contrabass saxophone.

    ..  container:: example

        >>> staff = abjad.Staff("c'4 d'4 e'4 fs'4")
        >>> contrabass_saxophone = abjad.ContrabassSaxophone()
        >>> abjad.attach(contrabass_saxophone, staff[0])
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                c'4
                d'4
                e'4
                fs'4
            }

    """

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(
        self,
        *,
        name='contrabass saxophone',
        short_name='cbass. sax.',
        markup=None,
        short_markup=None,
        allowable_clefs=None,
        context=None,
        middle_c_sounding_pitch='Eb1',
        pitch_range='[C1, Ab3]',
        ):
        Instrument.__init__(
            self,
            name=name,
            short_name=short_name,
            markup=markup,
            short_markup=short_markup,
            allowable_clefs=allowable_clefs,
            context=context,
            middle_c_sounding_pitch=middle_c_sounding_pitch,
            pitch_range=pitch_range,
            )


class Contrabassoon(Instrument):
    r"""
    Contrabassoon.

    ..  container:: example

        >>> staff = abjad.Staff("c'4 d'4 e'4 fs'4")
        >>> clef = abjad.Clef('bass')
        >>> abjad.attach(clef, staff[0])
        >>> contrabassoon = abjad.Contrabassoon()
        >>> abjad.attach(contrabassoon, staff[0])
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                \clef "bass"
                c'4
                d'4
                e'4
                fs'4
            }

    """

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(
        self,
        *,
        name='contrabassoon',
        short_name='contrabsn.',
        markup=None,
        short_markup=None,
        allowable_clefs=('bass',),
        context=None,
        middle_c_sounding_pitch='C3',
        pitch_range='[Bb0, Bb4]',
        ):
        Instrument.__init__(
            self,
            name=name,
            short_name=short_name,
            markup=markup,
            short_markup=short_markup,
            allowable_clefs=allowable_clefs,
            context=context,
            middle_c_sounding_pitch=middle_c_sounding_pitch,
            pitch_range=pitch_range,
            )


class EnglishHorn(Instrument):
    r"""
    English horn.

    ..  container:: example

        >>> staff = abjad.Staff("c'4 d'4 e'4 fs'4")
        >>> english_horn = abjad.EnglishHorn()
        >>> abjad.attach(english_horn, staff[0])
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                c'4
                d'4
                e'4
                fs'4
            }

    """

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(
        self,
        *,
        name='English horn',
        short_name='Eng. hn.',
        markup=None,
        short_markup=None,
        allowable_clefs=None,
        context=None,
        middle_c_sounding_pitch='F3',
        pitch_range='[E3, C6]',
        ):
        Instrument.__init__(
            self,
            name=name,
            short_name=short_name,
            markup=markup,
            short_markup=short_markup,
            allowable_clefs=allowable_clefs,
            context=context,
            middle_c_sounding_pitch=middle_c_sounding_pitch,
            pitch_range=pitch_range,
            )


class Flute(Instrument):
    r"""
    Flute.

    ..  container:: example

        >>> staff = abjad.Staff("c'4 d'4 e'4 fs'4")
        >>> flute = abjad.Flute()
        >>> abjad.attach(flute, staff[0])
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                c'4
                d'4
                e'4
                fs'4
            }

    ..  container:: example

        Instrument markup can be hidden:

        >>> staff = abjad.Staff("c'4 d'4 e'4 fs'4")
        >>> flute = abjad.Flute()
        >>> abjad.attach(flute, staff[0])
        >>> abjad.show(staff) # doctest: +SKIP

        >>> abjad.f(staff)
        \new Staff
        {
            c'4
            d'4
            e'4
            fs'4
        }

        >>> for leaf in abjad.select(staff).leaves():
        ...     leaf, abjad.inspect(leaf).effective(abjad.Instrument)
        ...
        (Note("c'4"), Flute())
        (Note("d'4"), Flute())
        (Note("e'4"), Flute())
        (Note("fs'4"), Flute())

    """

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(
        self,
        *,
        name='flute',
        short_name='fl.',
        markup=None,
        short_markup=None,
        allowable_clefs=None,
        context=None,
        middle_c_sounding_pitch=None,
        pitch_range='[C4, D7]',
        primary=True,
        ):
        Instrument.__init__(
            self,
            name=name,
            short_name=short_name,
            markup=markup,
            short_markup=short_markup,
            allowable_clefs=allowable_clefs,
            context=context,
            middle_c_sounding_pitch=middle_c_sounding_pitch,
            pitch_range=pitch_range,
            primary=primary,
            )


class FrenchHorn(Instrument):
    r"""
    French horn.

    ..  container:: example

        >>> staff = abjad.Staff("c'4 d'4 e'4 fs'4")
        >>> french_horn = abjad.FrenchHorn()
        >>> abjad.attach(french_horn, staff[0])
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                c'4
                d'4
                e'4
                fs'4
            }

    """

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(
        self,
        *,
        name='horn',
        short_name='hn.',
        markup=None,
        short_markup=None,
        allowable_clefs=('bass', 'treble'),
        context=None,
        middle_c_sounding_pitch='F3',
        pitch_range='[B1, F5]',
        primary=True,
        ):
        Instrument.__init__(
            self,
            name=name,
            short_name=short_name,
            markup=markup,
            short_markup=short_markup,
            allowable_clefs=allowable_clefs,
            context=context,
            middle_c_sounding_pitch=middle_c_sounding_pitch,
            pitch_range=pitch_range,
            primary=primary,
            )


class Glockenspiel(Instrument):
    r"""
    Glockenspiel.

    ..  container:: example

        >>> staff = abjad.Staff("c'4 d'4 e'4 fs'4")
        >>> glockenspiel = abjad.Glockenspiel()
        >>> abjad.attach(glockenspiel, staff[0])
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                c'4
                d'4
                e'4
                fs'4
            }

    """

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(
        self,
        *,
        name='glockenspiel',
        short_name='gkspl.',
        markup=None,
        short_markup=None,
        allowable_clefs=None,
        context=None,
        middle_c_sounding_pitch='C6',
        pitch_range='[G5, C8]',
        ):
        Instrument.__init__(
            self,
            name=name,
            short_name=short_name,
            markup=markup,
            short_markup=short_markup,
            allowable_clefs=allowable_clefs,
            context=context,
            middle_c_sounding_pitch=middle_c_sounding_pitch,
            pitch_range=pitch_range,
            )


class Guitar(Instrument):
    r"""
    Guitar.

    ..  container:: example

        >>> staff = abjad.Staff("c'4 d'4 e'4 fs'4")
        >>> guitar = abjad.Guitar()
        >>> abjad.attach(guitar, staff[0])
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                c'4
                d'4
                e'4
                fs'4
            }

    """

    ### CLASS VARIABLES ###

    __slots__ = (
        '_default_tuning',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        *,
        name='guitar',
        short_name='gt.',
        markup=None,
        short_markup=None,
        allowable_clefs=None,
        context=None,
        default_tuning=('E2', 'A2', 'D3', 'G3', 'B3', 'E4'),
        middle_c_sounding_pitch='C3',
        pitch_range='[E2, E5]',
        primary=True,
        ):
        Instrument.__init__(
            self,
            name=name,
            short_name=short_name,
            markup=markup,
            short_markup=short_markup,
            allowable_clefs=allowable_clefs,
            context=context,
            middle_c_sounding_pitch=middle_c_sounding_pitch,
            pitch_range=pitch_range,
            primary=primary,
            )
        self._default_tuning = Tuning(default_tuning)

    ### PUBLIC PROPERTIES ###

    @property
    def default_tuning(self):
        """
        Gets guitar's default tuning.

        ..  container:: example

            >>> guitar = abjad.Guitar()
            >>> guitar.default_tuning
            Tuning(pitches=PitchSegment(['e,', 'a,', 'd', 'g', 'b', "e'"]))

        Returns tuning.
        """
        return self._default_tuning


class Harp(Instrument):
    r"""
    Harp.

    ..  container:: example

        >>> staff_group = abjad.StaffGroup(lilypond_type='PianoStaff')
        >>> staff_group.append(abjad.Staff("c'4 d'4 e'4 f'4"))
        >>> staff_group.append(abjad.Staff("c'2 b2"))
        >>> harp = abjad.Harp()
        >>> abjad.attach(harp, staff_group[0][0])
        >>> abjad.attach(abjad.Clef('bass'), staff_group[1][0])
        >>> abjad.show(staff_group) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff_group)
            \new PianoStaff
            <<
                \new Staff
                {
                    c'4
                    d'4
                    e'4
                    f'4
                }
                \new Staff
                {
                    \clef "bass"
                    c'2
                    b2
                }
            >>

    The harp targets piano staff context by default.
    """

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(
        self,
        *,
        name='harp',
        short_name='hp.',
        markup=None,
        short_markup=None,
        allowable_clefs=('treble', 'bass'),
        context='StaffGroup',
        middle_c_sounding_pitch=None,
        pitch_range='[B0, G#7]',
        primary=True,
        ):
        Instrument.__init__(
            self,
            name=name,
            short_name=short_name,
            markup=markup,
            short_markup=short_markup,
            allowable_clefs=allowable_clefs,
            context=context,
            middle_c_sounding_pitch=middle_c_sounding_pitch,
            pitch_range=pitch_range,
            primary=primary,
            )


class Harpsichord(Instrument):
    r"""
    Harpsichord.

    ..  container:: example

        >>> upper_staff = abjad.Staff("c'4 d'4 e'4 f'4")
        >>> lower_staff = abjad.Staff("c'2 b2")
        >>> staff_group = abjad.StaffGroup(
        ...     [upper_staff, lower_staff],
        ...     lilypond_type='PianoStaff',
        ...     )
        >>> harpsichord = abjad.Harpsichord()
        >>> abjad.attach(harpsichord, staff_group[0][0])
        >>> abjad.attach(abjad.Clef('bass'), lower_staff[0])
        >>> abjad.show(staff_group) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff_group)
            \new PianoStaff
            <<
                \new Staff
                {
                    c'4
                    d'4
                    e'4
                    f'4
                }
                \new Staff
                {
                    \clef "bass"
                    c'2
                    b2
                }
            >>

    The harpsichord targets piano staff context by default.
    """

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(
        self,
        *,
        name='harpsichord',
        short_name='hpschd.',
        markup=None,
        short_markup=None,
        allowable_clefs=('treble', 'bass'),
        context='StaffGroup',
        middle_c_sounding_pitch=None,
        pitch_range='[C2, C7]',
        primary=True,
        ):
        Instrument.__init__(
            self,
            name=name,
            short_name=short_name,
            markup=markup,
            short_markup=short_markup,
            allowable_clefs=allowable_clefs,
            context=context,
            middle_c_sounding_pitch=middle_c_sounding_pitch,
            pitch_range=pitch_range,
            primary=primary,
            )


class Marimba(Instrument):
    r"""
    Marimba.

    ..  container:: example

        >>> staff = abjad.Staff("c'4 d'4 e'4 fs'4")
        >>> marimba = abjad.Marimba()
        >>> abjad.attach(marimba, staff[0])
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                c'4
                d'4
                e'4
                fs'4
            }

    """

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(
        self,
        *,
        name='marimba',
        short_name='mb.',
        markup=None,
        short_markup=None,
        allowable_clefs=('treble', 'bass'),
        context=None,
        middle_c_sounding_pitch=None,
        pitch_range='[F2, C7]',
        ):
        Instrument.__init__(
            self,
            name=name,
            short_name=short_name,
            markup=markup,
            short_markup=short_markup,
            allowable_clefs=allowable_clefs,
            context=context,
            middle_c_sounding_pitch=middle_c_sounding_pitch,
            pitch_range=pitch_range,
            )


class MezzoSopranoVoice(Instrument):
    r"""
    Mezzo-soprano voice.

    ..  container:: example


        >>> staff = abjad.Staff("c''4 d''4 e''4 fs''4")
        >>> mezzo_soprano = abjad.MezzoSopranoVoice()
        >>> abjad.attach(mezzo_soprano, staff[0])
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                c''4
                d''4
                e''4
                fs''4
            }

    """

    ### CLASS VARIABLES ###

    __slots__ = ()

    performer_abbreviation = 'ms.'

    ### INITIALIZER ###

    def __init__(
        self,
        *,
        name='mezzo-soprano',
        short_name='mezz.',
        markup=None,
        short_markup=None,
        allowable_clefs=None,
        context=None,
        middle_c_sounding_pitch=None,
        pitch_range='[A3, C6]',
        primary=True,
        ):
        Instrument.__init__(
            self,
            name=name,
            short_name=short_name,
            markup=markup,
            short_markup=short_markup,
            allowable_clefs=allowable_clefs,
            context=context,
            pitch_range=pitch_range,
            primary=primary,
            )


class Oboe(Instrument):
    r"""
    Oboe.

    ..  container:: example

        >>> staff = abjad.Staff("c'4 d'4 e'4 fs'4")
        >>> oboe = abjad.Oboe()
        >>> abjad.attach(oboe, staff[0])
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                c'4
                d'4
                e'4
                fs'4
            }

    """

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(
        self,
        *,
        name='oboe',
        short_name='ob.',
        markup=None,
        short_markup=None,
        allowable_clefs=None,
        context=None,
        middle_c_sounding_pitch=None,
        pitch_range='[Bb3, A6]',
        primary=True,
        ):
        Instrument.__init__(
            self,
            name=name,
            short_name=short_name,
            markup=markup,
            short_markup=short_markup,
            allowable_clefs=allowable_clefs,
            context=context,
            middle_c_sounding_pitch=middle_c_sounding_pitch,
            pitch_range=pitch_range,
            primary=primary,
            )


class Percussion(Instrument):
    r"""
    Percussion instrument.

    ..  container:: example

        >>> staff = abjad.Staff("c'4 d'4 e'4 fs'4")
        >>> percussion = abjad.Percussion()
        >>> abjad.attach(percussion, staff[0])
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                c'4
                d'4
                e'4
                fs'4
            }

    """

    ### CLASS VARIABLES ###

    __slots__ = ()

    known_percussion = list(sorted(set([
        'agogô',
        'anvil',
        'bass drum',
        'bongo drums',
        'cabasa',
        'cajón',
        'castanets',
        'caxixi',
        'claves',
        'conga drums',
        'cowbell',
        'crotales',
        'cuíca',
        'djembe',
        'finger cymbals',
        'flexatone',
        'frame drum',
        'gong',
        'güiro',
        'hand-held stones',
        'jawbone',
        'maracas',
        'ratchet',
        'rattle',
        'sand blocks',
        'scraped slate',
        'siren',
        'slapstick',
        'slide whistle',
        'snare drum',
        'sponges',
        'suspended cymbal',
        'steel drums',
        'tam-tam',
        'tambourine',
        'temple blocks',
        'thunder machine',
        'thundersheet',
        'toms',
        'tubular bells',
        'triangle',
        'vibraslap',
        'whistle',
        'wind chime',
        'wind machine',
        'wood blocks',
        'wood planks',
        ])))

    ### INITIALIZER ###

    def __init__(
        self,
        *,
        name='percussion',
        short_name='perc.',
        markup=None,
        short_markup=None,
        allowable_clefs=('percussion',),
        context=None,
        middle_c_sounding_pitch=None,
        pitch_range=None,
        ):
        Instrument.__init__(
            self,
            name=name,
            short_name=short_name,
            markup=markup,
            short_markup=short_markup,
            allowable_clefs=allowable_clefs,
            context=context,
            middle_c_sounding_pitch=middle_c_sounding_pitch,
            pitch_range=pitch_range,
            )


class Piano(Instrument):
    r"""
    Piano.

    ..  container:: example

        >>> staff_group = abjad.StaffGroup(lilypond_type='PianoStaff')
        >>> staff_group.append(abjad.Staff("c'4 d'4 e'4 f'4"))
        >>> staff_group.append(abjad.Staff("c'2 b2"))
        >>> piano = abjad.Piano()
        >>> abjad.attach(piano, staff_group[0][0])
        >>> abjad.attach(abjad.Clef('bass'), staff_group[1][0])
        >>> abjad.show(staff_group) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff_group)
            \new PianoStaff
            <<
                \new Staff
                {
                    c'4
                    d'4
                    e'4
                    f'4
                }
                \new Staff
                {
                    \clef "bass"
                    c'2
                    b2
                }
            >>

    The piano targets piano staff context by default.
    """

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(
        self,
        *,
        name='piano',
        short_name='pf.',
        markup=None,
        short_markup=None,
        allowable_clefs=('treble', 'bass'),
        context='StaffGroup',
        middle_c_sounding_pitch=None,
        pitch_range='[A0, C8]',
        primary=True,
        ):
        Instrument.__init__(
            self,
            name=name,
            short_name=short_name,
            markup=markup,
            short_markup=short_markup,
            allowable_clefs=allowable_clefs,
            context=context,
            middle_c_sounding_pitch=middle_c_sounding_pitch,
            pitch_range=pitch_range,
            primary=primary,
            )


class Piccolo(Instrument):
    r"""
    Piccolo.

    ..  container:: example

        >>> staff = abjad.Staff("c'4 d'4 e'4 fs'4")
        >>> piccolo = abjad.Piccolo()
        >>> abjad.attach(piccolo, staff[0])
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                c'4
                d'4
                e'4
                fs'4
            }

    """

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(
        self,
        *,
        name='piccolo',
        short_name='picc.',
        markup=None,
        short_markup=None,
        allowable_clefs=None,
        context=None,
        middle_c_sounding_pitch='C5',
        pitch_range='[D5, C8]',
        ):
        Instrument.__init__(
            self,
            name=name,
            short_name=short_name,
            markup=markup,
            short_markup=short_markup,
            allowable_clefs=allowable_clefs,
            context=context,
            middle_c_sounding_pitch=middle_c_sounding_pitch,
            pitch_range=pitch_range,
            )


class SopraninoSaxophone(Instrument):
    r"""
    Sopranino saxophone.

    ..  container:: example

        >>> staff = abjad.Staff("c'4 d'4 e'4 fs'4")
        >>> sopranino_saxophone = abjad.SopraninoSaxophone()
        >>> abjad.attach(sopranino_saxophone, staff[0])
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                c'4
                d'4
                e'4
                fs'4
            }

    """

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(
        self,
        *,
        name='sopranino saxophone',
        short_name='sopranino sax.',
        markup=None,
        short_markup=None,
        allowable_clefs=None,
        context=None,
        middle_c_sounding_pitch='Eb4',
        pitch_range='[Db4, F#6]',
        ):
        Instrument.__init__(
            self,
            name=name,
            short_name=short_name,
            markup=markup,
            short_markup=short_markup,
            allowable_clefs=allowable_clefs,
            context=context,
            middle_c_sounding_pitch=middle_c_sounding_pitch,
            pitch_range=pitch_range,
            )


class SopranoSaxophone(Instrument):
    r"""
    Soprano saxophone.

    ..  container:: example

        >>> staff = abjad.Staff("c'4 d'4 e'4 fs'4")
        >>> soprano_saxophone = abjad.SopranoSaxophone()
        >>> abjad.attach(soprano_saxophone, staff[0])
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                c'4
                d'4
                e'4
                fs'4
            }

    """

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(
        self,
        *,
        name='soprano saxophone',
        short_name='sop. sax.',
        markup=None,
        short_markup=None,
        allowable_clefs=None,
        context=None,
        middle_c_sounding_pitch='Bb3',
        pitch_range='[Ab3, E6]',
        ):
        Instrument.__init__(
            self,
            name=name,
            short_name=short_name,
            markup=markup,
            short_markup=short_markup,
            allowable_clefs=allowable_clefs,
            context=context,
            middle_c_sounding_pitch=middle_c_sounding_pitch,
            pitch_range=pitch_range,
            )


class SopranoVoice(Instrument):
    r"""
    Soprano voice.

    ..  container:: example

        >>> staff = abjad.Staff("c''4 d''4 e''4 fs''4")
        >>> soprano = abjad.SopranoVoice()
        >>> abjad.attach(soprano, staff[0])
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                c''4
                d''4
                e''4
                fs''4
            }

    """

    ### CLASS VARIABLES ###

    __slots__ = ()

    performer_abbreviation = 'sop.'

    ### INITIALIZER ###

    def __init__(
        self,
        *,
        name='soprano',
        short_name='sop.',
        markup=None,
        short_markup=None,
        allowable_clefs=None,
        context=None,
        middle_c_sounding_pitch=None,
        pitch_range='[C4, E6]',
        primary=True,
        ):
        Instrument.__init__(
            self,
            name=name,
            short_name=short_name,
            markup=markup,
            short_markup=short_markup,
            allowable_clefs=allowable_clefs,
            context=context,
            middle_c_sounding_pitch=middle_c_sounding_pitch,
            pitch_range=pitch_range,
            primary=primary,
            )


class TenorSaxophone(Instrument):
    r"""
    Tenor saxophone.

    ..  container:: example

        >>> staff = abjad.Staff("c'4 d'4 e'4 fs'4")
        >>> tenor_saxophone = abjad.TenorSaxophone()
        >>> abjad.attach(tenor_saxophone, staff[0])
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                c'4
                d'4
                e'4
                fs'4
            }

    """

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(
        self,
        *,
        name='tenor saxophone',
        short_name='ten. sax.',
        markup=None,
        short_markup=None,
        allowable_clefs=None,
        context=None,
        middle_c_sounding_pitch='Bb2',
        pitch_range='[Ab2, E5]',
        ):
        Instrument.__init__(
            self,
            name=name,
            short_name=short_name,
            markup=markup,
            short_markup=short_markup,
            allowable_clefs=allowable_clefs,
            context=context,
            middle_c_sounding_pitch=middle_c_sounding_pitch,
            pitch_range=pitch_range,
            )


class TenorTrombone(Instrument):
    r"""
    Tenor trombone.

    ..  container:: example

        >>> staff = abjad.Staff("c'4 d'4 e'4 fs'4")
        >>> clef = abjad.Clef('bass')
        >>> abjad.attach(clef, staff[0])
        >>> tenor_trombone = abjad.TenorTrombone()
        >>> abjad.attach(tenor_trombone, staff[0])
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                \clef "bass"
                c'4
                d'4
                e'4
                fs'4
            }

    """

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(
        self,
        *,
        name='tenor trombone',
        short_name='ten. trb.',
        markup=None,
        short_markup=None,
        allowable_clefs=('tenor', 'bass'),
        context=None,
        middle_c_sounding_pitch=None,
        pitch_range='[E2, Eb5]',
        primary=True,
        ):
        Instrument.__init__(
            self,
            name=name,
            short_name=short_name,
            markup=markup,
            short_markup=short_markup,
            allowable_clefs=allowable_clefs,
            context=context,
            pitch_range=pitch_range,
            primary=primary,
            )


class TenorVoice(Instrument):
    r"""
    Tenor voice.

    ..  container:: example

        >>> staff = abjad.Staff("c'4 d'4 e'4 fs'4")
        >>> tenor = abjad.TenorVoice()
        >>> abjad.attach(tenor, staff[0])
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                c'4
                d'4
                e'4
                fs'4
            }

    """

    ### CLASS VARIABLES ###

    __slots__ = ()

    performer_abbreviation = 'ten.'

    ### INITIALIZER ###

    def __init__(
        self,
        *,
        name='tenor',
        short_name='ten.',
        markup=None,
        short_markup=None,
        allowable_clefs=None,
        context=None,
        middle_c_sounding_pitch=None,
        pitch_range='[C3, D5]',
        primary=True,
        ):
        Instrument.__init__(
            self,
            name=name,
            short_name=short_name,
            markup=markup,
            short_markup=short_markup,
            allowable_clefs=allowable_clefs,
            context=context,
            middle_c_sounding_pitch=middle_c_sounding_pitch,
            pitch_range=pitch_range,
            primary=primary,
            )


class Trumpet(Instrument):
    r"""
    Trumpet.

    ..  container:: example

        >>> staff = abjad.Staff("c'4 d'4 e'4 fs'4")
        >>> trumpet = abjad.Trumpet()
        >>> abjad.attach(trumpet, staff[0])
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                c'4
                d'4
                e'4
                fs'4
            }

    """

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(
        self,
        *,
        name='trumpet',
        short_name='tp.',
        markup=None,
        short_markup=None,
        allowable_clefs=None,
        context=None,
        middle_c_sounding_pitch=None,
        pitch_range='[F#3, D6]',
        primary=True,
        ):
        Instrument.__init__(
            self,
            name=name,
            short_name=short_name,
            markup=markup,
            short_markup=short_markup,
            allowable_clefs=allowable_clefs,
            context=context,
            middle_c_sounding_pitch=middle_c_sounding_pitch,
            pitch_range=pitch_range,
            primary=primary,
            )


class Tuba(Instrument):
    r"""
    Tuba.

    ..  container:: example

        >>> staff = abjad.Staff("c'4 d'4 e'4 fs'4")
        >>> clef = abjad.Clef('bass')
        >>> abjad.attach(clef, staff[0])
        >>> tuba = abjad.Tuba()
        >>> abjad.attach(tuba, staff[0])
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                \clef "bass"
                c'4
                d'4
                e'4
                fs'4
            }

    """

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(
        self,
        *,
        name='tuba',
        short_name='tb.',
        markup=None,
        short_markup=None,
        allowable_clefs=('bass',),
        context=None,
        middle_c_sounding_pitch=None,
        pitch_range='[D1, F4]',
        primary=True,
        ):
        Instrument.__init__(
            self,
            name=name,
            short_name=short_name,
            markup=markup,
            short_markup=short_markup,
            allowable_clefs=allowable_clefs,
            context=context,
            middle_c_sounding_pitch=middle_c_sounding_pitch,
            pitch_range=pitch_range,
            primary=primary,
            )


class Vibraphone(Instrument):
    r"""
    Vibraphone.

    ..  container:: example

        >>> staff = abjad.Staff("c'4 d'4 e'4 fs'4")
        >>> vibraphone = abjad.Vibraphone()
        >>> abjad.attach(vibraphone, staff[0])
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                c'4
                d'4
                e'4
                fs'4
            }

    """

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(
        self,
        *,
        name='vibraphone',
        short_name='vibr.',
        markup=None,
        short_markup=None,
        allowable_clefs=None,
        context=None,
        middle_c_sounding_pitch=None,
        pitch_range='[F3, F6]',
        ):
        Instrument.__init__(
            self,
            name=name,
            short_name=short_name,
            markup=markup,
            short_markup=short_markup,
            allowable_clefs=allowable_clefs,
            context=context,
            middle_c_sounding_pitch=middle_c_sounding_pitch,
            pitch_range=pitch_range,
            )


class Viola(Instrument):
    r"""
    Viola.

    ..  container:: example

        >>> staff = abjad.Staff("c'4 d'4 e'4 fs'4")
        >>> clef = abjad.Clef('alto')
        >>> abjad.attach(clef, staff[0])
        >>> viola = abjad.Viola()
        >>> abjad.attach(viola, staff[0])
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                \clef "alto"
                c'4
                d'4
                e'4
                fs'4
            }

    """

    ### CLASS VARIABLES ###

    __slots__ = (
        '_default_tuning',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        *,
        name='viola',
        short_name='va.',
        markup=None,
        short_markup=None,
        allowable_clefs=('alto', 'treble'),
        context=None,
        default_tuning=('C3', 'G3', 'D4', 'A4'),
        middle_c_sounding_pitch=None,
        pitch_range='[C3, D6]',
        primary=True,
        ):
        Instrument.__init__(
            self,
            name=name,
            short_name=short_name,
            markup=markup,
            short_markup=short_markup,
            allowable_clefs=allowable_clefs,
            context=context,
            middle_c_sounding_pitch=middle_c_sounding_pitch,
            pitch_range=pitch_range,
            primary=primary,
            )
        self._default_tuning = Tuning(default_tuning)

    ### PUBLIC PROPERTIES ###

    @property
    def default_tuning(self):
        """
        Gets viola's default tuning.

        ..  container:: example

            >>> viola = abjad.Viola()
            >>> viola.default_tuning
            Tuning(pitches=PitchSegment(['c', 'g', "d'", "a'"]))

        Returns tuning.
        """
        return self._default_tuning


class Violin(Instrument):
    r"""
    Violin.

    ..  container:: example

        >>> staff = abjad.Staff("c'4 d'4 e'4 fs'4")
        >>> violin = abjad.Violin()
        >>> abjad.attach(violin, staff[0])
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                c'4
                d'4
                e'4
                fs'4
            }

    """

    ### CLASS VARIABLES ###

    __slots__ = (
        '_default_tuning',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        *,
        name='violin',
        short_name='vn.',
        markup=None,
        short_markup=None,
        allowable_clefs=None,
        context=None,
        default_tuning=('G3', 'D4', 'A4', 'E5'),
        middle_c_sounding_pitch=None,
        pitch_range='[G3, G7]',
        primary=True,
        ):
        Instrument.__init__(
            self,
            name=name,
            short_name=short_name,
            markup=markup,
            short_markup=short_markup,
            allowable_clefs=allowable_clefs,
            context=context,
            middle_c_sounding_pitch=middle_c_sounding_pitch,
            pitch_range=pitch_range,
            primary=primary,
            )
        self._default_tuning = Tuning(default_tuning)

    ### PUBLIC PROPERTIES ###

    @property
    def default_tuning(self):
        """
        Gets violin's default tuning.

        ..  container:: example

            >>> violin = abjad.Violin()
            >>> violin.default_tuning
            Tuning(pitches=PitchSegment(['g', "d'", "a'", "e''"]))

        Returns tuning.
        """
        return self._default_tuning


class Xylophone(Instrument):
    r"""
    Xylphone.

    ..  container:: example

        >>> staff = abjad.Staff("c'4 d'4 e'4 fs'4")
        >>> xylophone = abjad.Xylophone()
        >>> abjad.attach(xylophone, staff[0])
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                c'4
                d'4
                e'4
                fs'4
            }

    """

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(
        self,
        *,
        name='xylophone',
        short_name='xyl.',
        markup=None,
        short_markup=None,
        allowable_clefs=None,
        context=None,
        middle_c_sounding_pitch='C5',
        pitch_range='[C4, C7]',
        ):
        Instrument.__init__(
            self,
            name=name,
            short_name=short_name,
            markup=markup,
            short_markup=short_markup,
            allowable_clefs=allowable_clefs,
            context=context,
            middle_c_sounding_pitch=middle_c_sounding_pitch,
            pitch_range=pitch_range,
            )



def offset_happens_during_timespan(timespan=None, offset=None, hold=False):
    """
    Makes time relation indicating that ``offset`` happens during ``timespan``.

    ..  container:: example

        >>> relation = abjad.timespans.offset_happens_during_timespan()
        >>> abjad.f(relation)
        abjad.timespans.OffsetTimespanTimeRelation(
            inequality=abjad.timespans.CompoundInequality(
                [
                    abjad.TimespanInequality('timespan.start <= offset'),
                    abjad.TimespanInequality('offset < timespan.stop'),
                    ],
                logical_operator='and',
                ),
            )

    Returns time relation or boolean.
    """
    from abjad import timespans

    inequality = timespans.CompoundInequality([
        'timespan.start <= offset',
        'offset < timespan.stop',
        ])

    time_relation = timespans.OffsetTimespanTimeRelation(
        inequality,
        timespan=timespan,
        offset=offset)

    if time_relation.is_fully_loaded and not hold:
        return time_relation()
    else:
        return time_relation
